@@ -14,7 +14,8 @@
 
 use std::io::Write as _;
 
-use jj_cli::cli_util::{CliRunner, CommandError, CommandHelper};
+use jj_cli::cli_util::{CliRunner, CommandHelper, RevisionArg};
+use jj_cli::command_error::CommandError;
 use jj_cli::ui::Ui;
 
 #[derive(clap::Parser, Clone, Debug)]
@@ -27,7 +28,7 @@ enum CustomCommand {
 struct FrobnicateArgs {
     /// The revision to frobnicate
     #[arg(default_value = "@")]
-    revision: String,
+    revision: RevisionArg,
 }
 
 fn run_custom_command(
@@ -38,16 +39,16 @@ fn run_custom_command(
     match command {
         CustomCommand::Frobnicate(args) => {
             let mut workspace_command = command_helper.workspace_helper(ui)?;
-            let commit = workspace_command.resolve_single_rev(&args.revision, ui)?;
-            let mut tx = workspace_command.start_transaction("Frobnicate");
+            let commit = workspace_command.resolve_single_rev(&args.revision)?;
+            let mut tx = workspace_command.start_transaction();
             let new_commit = tx
                 .mut_repo()
                 .rewrite_commit(command_helper.settings(), &commit)
                 .set_description("Frobnicated!")
                 .write()?;
-            tx.finish(ui)?;
+            tx.finish(ui, "Frobnicate")?;
             writeln!(
-                ui.stderr(),
+                ui.status(),
                 "Frobnicated revision: {}",
                 workspace_command.format_commit_summary(&new_commit)
             )?;
