@@ -17,8 +17,11 @@ use jj_lib::repo::Repo;
 use jj_lib::rewrite::merge_commit_trees;
 use tracing::instrument;
 
-use crate::cli_util::{join_message_paragraphs, user_error, CommandError, CommandHelper};
-use crate::description_util::{description_template_for_commit, edit_description};
+use crate::cli_util::CommandHelper;
+use crate::command_error::{user_error, CommandError};
+use crate::description_util::{
+    description_template_for_commit, edit_description, join_message_paragraphs,
+};
 use crate::ui::Ui;
 
 /// Update the description and create a new change on top.
@@ -28,6 +31,9 @@ pub(crate) struct CommitArgs {
     /// Interactively choose which changes to include in the first commit
     #[arg(short, long)]
     interactive: bool,
+    /// Specify diff editor to be used (implies --interactive)
+    #[arg(long, value_name = "NAME")]
+    tool: Option<String>,
     /// The change description to use (don't open editor)
     #[arg(long = "message", short, value_name = "MESSAGE")]
     message_paragraphs: Vec<String>,
@@ -48,7 +54,12 @@ pub(crate) fn cmd_commit(
         .get_wc_commit_id()
         .ok_or_else(|| user_error("This command requires a working copy"))?;
     let commit = workspace_command.repo().store().get_commit(commit_id)?;
-    let matcher = workspace_command.matcher_from_values(&args.paths)?;
+    let matcher = workspace_command
+        .parse_file_patterns(&args.paths)?
+        .to_matcher();
+    let advanceable_branches = workspace_command.get_advanceable_branches(commit.parent_ids())?;
+    let diff_selector =
+        workspace_command.diff_selector(ui, args.tool.as_deref(), args.interactive)?;
     let mut tx = workspace_command.start_transaction();
     let base_tree = merge_commit_trees(tx.repo(), &commit.parents())?;
     let instructions = format!(
@@ -61,18 +72,16 @@ new working-copy commit.
 ",
         tx.format_commit_summary(&commit)
     );
-    let tree_id = tx.select_diff(
-        ui,
+    let tree_id = diff_selector.select(
         &base_tree,
         &commit.tree()?,
         matcher.as_ref(),
-        &instructions,
-        args.interactive,
+        Some(&instructions),
     )?;
     let middle_tree = tx.repo().store().get_root_tree(&tree_id)?;
     if !args.paths.is_empty() && middle_tree.id() == base_tree.id() {
         writeln!(
-            ui.warning(),
+            ui.warning_default(),
             "The given paths do not match any file: {}",
             args.paths.join(" ")
         )?;
@@ -113,6 +122,10 @@ new working-copy commit.
                 commit.tree_id().clone(),
             )
             .write()?;
+
+        // Does nothing if there's no branches to advance.
+        tx.advance_branches(advanceable_branches, new_commit.id());
+
         for workspace_id in workspace_ids {
             tx.mut_repo().edit(workspace_id, &new_wc_commit).unwrap();
         }
