@@ -19,47 +19,51 @@ use jj_lib::commit::Commit;
 use jj_lib::repo::Repo;
 use jj_lib::revset::{RevsetExpression, RevsetIteratorExt};
 
-use crate::cli_util::{
-    short_commit_hash, user_error, CommandError, CommandHelper, WorkspaceCommandHelper,
-};
+use crate::cli_util::{short_commit_hash, CommandHelper, WorkspaceCommandHelper};
+use crate::command_error::{user_error, CommandError};
 use crate::ui::Ui;
 
-/// Move the current working copy commit to the next child revision in the
-/// repository.
+/// Move the working-copy commit to the child revision
 ///
-/// The command moves you to the next child in a linear fashion.
+/// The command creates a new empty working copy revision that is the child of a
+/// descendant `offset` revisions ahead of the parent of the current working
+/// copy.
+///
+/// For example, when the offset is 1:
 ///
 /// ```text
-/// D      D @
-/// |      |/
-/// C @ => C
-/// |/     |
-/// B      B
+/// D        D @
+/// |        |/
+/// C @  =>  C
+/// |/       |
+/// B        B
 /// ```
 ///
-/// If `--edit` is passed, it will move you directly to the child
-/// revision.
+/// If `--edit` is passed, the working copy revision is changed to the child of
+/// the current working copy revision.
 ///
 /// ```text
-/// D    D
-/// |    |
-/// C    C
-/// |    |
-/// B => @
-/// |    |
-/// @    A
+/// D        D
+/// |        |
+/// C        C
+/// |        |
+/// B   =>   @
+/// |        |
+/// @        A
 /// ```
+/// If your working-copy commit already has visible children, then `--edit` is
+/// implied.
 #[derive(clap::Args, Clone, Debug)]
 #[command(verbatim_doc_comment)]
 pub(crate) struct NextArgs {
-    /// How many revisions to move forward. By default advances to the next
-    /// child.
+    /// How many revisions to move forward. Advances to the next child by
+    /// default.
     #[arg(default_value = "1")]
-    amount: u64,
+    offset: u64,
     /// Instead of creating a new working-copy commit on top of the target
     /// commit (like `jj new`), edit the target commit directly (like `jj
     /// edit`).
-    #[arg(long)]
+    #[arg(long, short)]
     edit: bool,
 }
 
@@ -71,10 +75,11 @@ pub fn choose_commit<'a>(
 ) -> Result<&'a Commit, CommandError> {
     writeln!(ui.stdout(), "ambiguous {cmd} commit, choose one to target:")?;
     let mut formatter = ui.stdout_formatter();
+    let template = workspace_command.commit_summary_template();
     let mut choices: Vec<String> = Default::default();
     for (i, commit) in commits.iter().enumerate() {
         write!(formatter, "{}: ", i + 1)?;
-        workspace_command.write_commit_summary(formatter.as_mut(), commit)?;
+        template.format(commit, formatter.as_mut())?;
         writeln!(formatter)?;
         choices.push(format!("{}", i + 1));
     }
@@ -100,51 +105,51 @@ pub(crate) fn cmd_next(
     args: &NextArgs,
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
-    let edit = args.edit;
-    let amount = args.amount;
     let current_wc_id = workspace_command
         .get_wc_commit_id()
         .ok_or_else(|| user_error("This command requires a working copy"))?;
-    let current_wc = workspace_command.repo().store().get_commit(current_wc_id)?;
-    let current_short = short_commit_hash(current_wc.id());
-    // If we're editing, start at the working-copy commit.
-    // Otherwise start from our direct parent.
-    let start_id = if edit {
-        current_wc_id
+    let edit = args.edit
+        || !workspace_command
+            .repo()
+            .view()
+            .heads()
+            .contains(current_wc_id);
+    let wc_revset = RevsetExpression::commit(current_wc_id.clone());
+    // If we're editing, start at the working-copy commit. Otherwise, start from
+    // its direct parent(s).
+    let target_revset = if edit {
+        wc_revset.descendants_at(args.offset)
     } else {
-        match current_wc.parent_ids() {
-            [parent_id] => parent_id,
-            _ => return Err(user_error("Cannot run `jj next` on a merge commit")),
-        }
+        wc_revset
+            .parents()
+            .descendants_at(args.offset)
+            // In previous versions we subtracted `wc_revset.descendants()`. That's
+            // unnecessary now that --edit is implied if `@` has descendants.
+            .minus(&wc_revset)
     };
-    let descendant_expression = RevsetExpression::commit(start_id.clone()).descendants_at(amount);
-    let target_expression = if edit {
-        descendant_expression
-    } else {
-        descendant_expression.minus(&RevsetExpression::commit(current_wc_id.clone()).descendants())
-    };
-    let targets: Vec<Commit> = target_expression
+    let targets: Vec<Commit> = target_revset
         .evaluate_programmatic(workspace_command.repo().as_ref())?
         .iter()
         .commits(workspace_command.repo().store())
-        .take(2)
         .try_collect()?;
     let target = match targets.as_slice() {
         [target] => target,
         [] => {
             // We found no descendant.
             return Err(user_error(format!(
-                "No descendant found {amount} commit{} forward",
-                if amount > 1 { "s" } else { "" }
+                "No descendant found {} commit{} forward",
+                args.offset,
+                if args.offset > 1 { "s" } else { "" }
             )));
         }
         commits => choose_commit(ui, &workspace_command, "next", commits)?,
     };
+    let current_short = short_commit_hash(current_wc_id);
     let target_short = short_commit_hash(target.id());
     // We're editing, just move to the target commit.
     if edit {
         // We're editing, the target must be rewritable.
-        workspace_command.check_rewritable([target])?;
+        workspace_command.check_rewritable([target.id()])?;
         let mut tx = workspace_command.start_transaction();
         tx.edit(target)?;
         tx.finish(
