@@ -18,7 +18,8 @@ use jj_lib::object_id::ObjectId;
 use jj_lib::rewrite::{merge_commit_trees, restore_tree};
 use tracing::instrument;
 
-use crate::cli_util::{user_error, CommandError, CommandHelper, RevisionArg};
+use crate::cli_util::{CommandHelper, RevisionArg};
+use crate::command_error::{user_error, CommandError};
 use crate::ui::Ui;
 
 /// Restore paths from another revision
@@ -85,22 +86,25 @@ pub(crate) fn cmd_restore(
         ));
     }
     if args.from.is_some() || args.to.is_some() {
-        to_commit = workspace_command.resolve_single_rev(args.to.as_deref().unwrap_or("@"), ui)?;
+        to_commit =
+            workspace_command.resolve_single_rev(args.to.as_ref().unwrap_or(&RevisionArg::AT))?;
         from_tree = workspace_command
-            .resolve_single_rev(args.from.as_deref().unwrap_or("@"), ui)?
+            .resolve_single_rev(args.from.as_ref().unwrap_or(&RevisionArg::AT))?
             .tree()?;
     } else {
-        to_commit =
-            workspace_command.resolve_single_rev(args.changes_in.as_deref().unwrap_or("@"), ui)?;
+        to_commit = workspace_command
+            .resolve_single_rev(args.changes_in.as_ref().unwrap_or(&RevisionArg::AT))?;
         from_tree = merge_commit_trees(workspace_command.repo().as_ref(), &to_commit.parents())?;
     }
-    workspace_command.check_rewritable([&to_commit])?;
+    workspace_command.check_rewritable([to_commit.id()])?;
 
-    let matcher = workspace_command.matcher_from_values(&args.paths)?;
+    let matcher = workspace_command
+        .parse_file_patterns(&args.paths)?
+        .to_matcher();
     let to_tree = to_commit.tree()?;
     let new_tree_id = restore_tree(&from_tree, &to_tree, matcher.as_ref())?;
     if &new_tree_id == to_commit.tree_id() {
-        writeln!(ui.stderr(), "Nothing changed.")?;
+        writeln!(ui.status(), "Nothing changed.")?;
     } else {
         let mut tx = workspace_command.start_transaction();
         let mut_repo = tx.mut_repo();
@@ -111,11 +115,13 @@ pub(crate) fn cmd_restore(
         // rebase_descendants early; otherwise `new_commit` would always have
         // a conflicted change id at this point.
         let num_rebased = tx.mut_repo().rebase_descendants(command.settings())?;
-        write!(ui.stderr(), "Created ")?;
-        tx.write_commit_summary(ui.stderr_formatter().as_mut(), &new_commit)?;
-        writeln!(ui.stderr())?;
-        if num_rebased > 0 {
-            writeln!(ui.stderr(), "Rebased {num_rebased} descendant commits")?;
+        if let Some(mut formatter) = ui.status_formatter() {
+            write!(formatter, "Created ")?;
+            tx.write_commit_summary(formatter.as_mut(), &new_commit)?;
+            writeln!(formatter)?;
+            if num_rebased > 0 {
+                writeln!(formatter, "Rebased {num_rebased} descendant commits")?;
+            }
         }
         tx.finish(ui, format!("restore into commit {}", to_commit.id().hex()))?;
     }
