@@ -13,15 +13,15 @@
 // limitations under the License.
 
 use itertools::Itertools;
-use jj_lib::matchers::EverythingMatcher;
 use jj_lib::repo::Repo;
+use jj_lib::revset::{RevsetExpression, RevsetFilterPredicate};
 use jj_lib::rewrite::merge_commit_trees;
 use tracing::instrument;
 
-use super::resolve;
-use crate::cli_util::{CommandError, CommandHelper};
-use crate::diff_util;
+use crate::cli_util::{print_conflicted_paths, CommandHelper};
+use crate::command_error::CommandError;
 use crate::ui::Ui;
+use crate::{diff_util, revset_util};
 
 /// Show high-level repo status
 ///
@@ -33,13 +33,17 @@ use crate::ui::Ui;
 ///  * Conflicted branches (see https://github.com/martinvonz/jj/blob/main/docs/branches.md)
 #[derive(clap::Args, Clone, Debug)]
 #[command(visible_alias = "st")]
-pub(crate) struct StatusArgs {}
+pub(crate) struct StatusArgs {
+    /// Restrict the status display to these paths
+    #[arg(value_hint = clap::ValueHint::AnyPath)]
+    paths: Vec<String>,
+}
 
 #[instrument(skip_all)]
 pub(crate) fn cmd_status(
     ui: &mut Ui,
     command: &CommandHelper,
-    _args: &StatusArgs,
+    args: &StatusArgs,
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
     let repo = workspace_command.repo();
@@ -47,6 +51,9 @@ pub(crate) fn cmd_status(
         .get_wc_commit_id()
         .map(|id| repo.store().get_commit(id))
         .transpose()?;
+    let matcher = workspace_command
+        .parse_file_patterns(&args.paths)?
+        .to_matcher();
     ui.request_pager();
     let mut formatter = ui.stdout_formatter();
     let formatter = formatter.as_mut();
@@ -55,37 +62,52 @@ pub(crate) fn cmd_status(
         let parent_tree = merge_commit_trees(repo.as_ref(), &wc_commit.parents())?;
         let tree = wc_commit.tree()?;
         if tree.id() == parent_tree.id() {
-            formatter.write_str("The working copy is clean\n")?;
+            writeln!(formatter, "The working copy is clean")?;
         } else {
-            formatter.write_str("Working copy changes:\n")?;
+            writeln!(formatter, "Working copy changes:")?;
             diff_util::show_diff_summary(
                 formatter,
                 &workspace_command,
-                parent_tree.diff_stream(&tree, &EverythingMatcher),
+                parent_tree.diff_stream(&tree, matcher.as_ref()),
             )?;
         }
 
+        // TODO: Conflicts should also be filtered by the `matcher`. See the related
+        // TODO on `MergedTree::conflicts()`.
         let conflicts = wc_commit.tree()?.conflicts().collect_vec();
         if !conflicts.is_empty() {
             writeln!(
                 formatter.labeled("conflict"),
                 "There are unresolved conflicts at these paths:"
             )?;
-            resolve::print_conflicted_paths(&conflicts, formatter, &workspace_command)?
+            print_conflicted_paths(&conflicts, formatter, &workspace_command)?
         }
 
-        formatter.write_str("Working copy : ")?;
-        formatter.with_label("working_copy", |fmt| {
-            workspace_command.write_commit_summary(fmt, wc_commit)
-        })?;
-        formatter.write_str("\n")?;
+        let template = workspace_command.commit_summary_template();
+        write!(formatter, "Working copy : ")?;
+        formatter.with_label("working_copy", |fmt| template.format(wc_commit, fmt))?;
+        writeln!(formatter)?;
         for parent in wc_commit.parents() {
-            formatter.write_str("Parent commit: ")?;
-            workspace_command.write_commit_summary(formatter, &parent)?;
-            formatter.write_str("\n")?;
+            write!(formatter, "Parent commit: ")?;
+            template.format(&parent, formatter)?;
+            writeln!(formatter)?;
         }
+
+        let wc_revset = RevsetExpression::commit(wc_commit.id().clone());
+        // Ancestors with conflicts, excluding the current working copy commit.
+        let ancestors_conflicts = workspace_command
+            .attach_revset_evaluator(
+                RevsetExpression::filter(RevsetFilterPredicate::HasConflict)
+                    .intersection(&wc_revset.parents().ancestors())
+                    .minus(&revset_util::parse_immutable_expression(
+                        &workspace_command.revset_parse_context(),
+                    )?),
+            )?
+            .evaluate_to_commit_ids()?
+            .collect();
+        workspace_command.report_repo_conflicts(formatter, repo, ancestors_conflicts)?;
     } else {
-        formatter.write_str("No working copy\n")?;
+        writeln!(formatter, "No working copy")?;
     }
 
     let conflicted_local_branches = repo
