@@ -13,6 +13,7 @@
 // limitations under the License.
 
 mod builtin;
+mod diff_working_copies;
 mod external;
 
 use std::sync::Arc;
@@ -30,7 +31,8 @@ use pollster::FutureExt;
 use thiserror::Error;
 
 use self::builtin::{edit_diff_builtin, edit_merge_builtin, BuiltinToolError};
-use self::external::{edit_diff_external, DiffCheckoutError, ExternalToolError};
+use self::diff_working_copies::DiffCheckoutError;
+use self::external::{edit_diff_external, ExternalToolError};
 pub use self::external::{generate_diff, ExternalMergeTool};
 use crate::config::CommandNameAndArgs;
 use crate::ui::Ui;
@@ -77,88 +79,36 @@ pub enum ConflictResolveError {
     #[error("The conflict at {path:?} has {sides} sides. At most 2 sides are supported.")]
     ConflictTooComplicated { path: RepoPathBuf, sides: usize },
     #[error(
-        "The output file is either unchanged or empty after the editor quit (run with --verbose \
-         to see the exact invocation)."
+        "The output file is either unchanged or empty after the editor quit (run with --debug to \
+         see the exact invocation)."
     )]
     EmptyOrUnchanged,
     #[error("Backend error")]
     Backend(#[from] jj_lib::backend::BackendError),
 }
 
-pub fn run_mergetool(
-    ui: &Ui,
-    tree: &MergedTree,
-    repo_path: &RepoPath,
-    settings: &UserSettings,
-) -> Result<MergedTreeId, ConflictResolveError> {
-    let conflict = match tree.path_value(repo_path).into_resolved() {
-        Err(conflict) => conflict,
-        Ok(Some(_)) => return Err(ConflictResolveError::NotAConflict(repo_path.to_owned())),
-        Ok(None) => return Err(ConflictResolveError::PathNotFound(repo_path.to_owned())),
-    };
-    let file_merge = conflict.to_file_merge().ok_or_else(|| {
-        let mut summary_bytes: Vec<u8> = vec![];
-        conflict
-            .describe(&mut summary_bytes)
-            .expect("Writing to an in-memory buffer should never fail");
-        ConflictResolveError::NotNormalFiles(
-            repo_path.to_owned(),
-            String::from_utf8_lossy(summary_bytes.as_slice()).to_string(),
-        )
-    })?;
-    // We only support conflicts with 2 sides (3-way conflicts)
-    if file_merge.num_sides() > 2 {
-        return Err(ConflictResolveError::ConflictTooComplicated {
-            path: repo_path.to_owned(),
-            sides: file_merge.num_sides(),
-        });
-    };
-    let content = extract_as_single_hunk(&file_merge, tree.store(), repo_path).block_on();
-
-    let editor = get_merge_tool_from_settings(ui, settings)?;
-    match editor {
-        MergeTool::Builtin => {
-            let tree_id = edit_merge_builtin(tree, repo_path, content).map_err(Box::new)?;
-            Ok(tree_id)
-        }
-        MergeTool::External(editor) => external::run_mergetool_external(
-            &editor, file_merge, content, repo_path, conflict, tree,
-        ),
-    }
-}
-
-pub fn edit_diff(
-    ui: &Ui,
-    left_tree: &MergedTree,
-    right_tree: &MergedTree,
-    matcher: &dyn Matcher,
-    instructions: &str,
-    base_ignores: Arc<GitIgnoreFile>,
-    settings: &UserSettings,
-) -> Result<MergedTreeId, DiffEditError> {
-    // Start a diff editor on the two directories.
-    let editor = get_diff_editor_from_settings(ui, settings)?;
-    match editor {
-        MergeTool::Builtin => {
-            let tree_id = edit_diff_builtin(left_tree, right_tree, matcher).map_err(Box::new)?;
-            Ok(tree_id)
-        }
-        MergeTool::External(editor) => edit_diff_external(
-            editor,
-            left_tree,
-            right_tree,
-            matcher,
-            instructions,
-            base_ignores,
-            settings,
-        ),
-    }
+#[derive(Debug, Error)]
+pub enum MergeToolConfigError {
+    #[error(transparent)]
+    Config(#[from] ConfigError),
+    #[error(
+        "To use `{tool_name}` as a merge tool, the config `merge-tools.{tool_name}.merge-args` \
+         must be defined (see docs for details)"
+    )]
+    MergeArgsNotConfigured { tool_name: String },
 }
 
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub enum MergeTool {
     Builtin,
-    External(ExternalMergeTool),
+    // Boxed because ExternalMergeTool is big compared to the Builtin variant.
+    External(Box<ExternalMergeTool>),
+}
+
+impl MergeTool {
+    fn external(tool: ExternalMergeTool) -> Self {
+        MergeTool::External(Box::new(tool))
+    }
 }
 
 /// Finds the appropriate tool for diff editing or merges
@@ -166,31 +116,40 @@ fn editor_args_from_settings(
     ui: &Ui,
     settings: &UserSettings,
     key: &str,
-) -> Result<CommandNameAndArgs, ExternalToolError> {
+) -> Result<CommandNameAndArgs, ConfigError> {
     // TODO: Make this configuration have a table of possible editors and detect the
     // best one here.
     if let Some(args) = settings.config().get(key).optional()? {
         Ok(args)
     } else {
         let default_editor = BUILTIN_EDITOR_NAME;
-        writeln!(
-            ui.hint(),
-            "Using default editor '{default_editor}'; you can change this by setting {key}"
-        )
-        .map_err(ExternalToolError::Io)?;
+        if let Some(mut writer) = ui.hint_default() {
+            writeln!(
+                writer,
+                "Using default editor '{default_editor}'; run `jj config set --user {key} \
+                 :builtin` to disable this message."
+            )
+            .ok();
+        }
         Ok(default_editor.into())
     }
 }
 
-/// Loads merge tool options from `[merge-tools.<name>]`.
-pub fn get_tool_config(
+/// Resolves builtin merge tool name or loads external tool options from
+/// `[merge-tools.<name>]`.
+fn get_tool_config(settings: &UserSettings, name: &str) -> Result<Option<MergeTool>, ConfigError> {
+    if name == BUILTIN_EDITOR_NAME {
+        Ok(Some(MergeTool::Builtin))
+    } else {
+        Ok(get_external_tool_config(settings, name)?.map(MergeTool::external))
+    }
+}
+
+/// Loads external diff/merge tool options from `[merge-tools.<name>]`.
+pub fn get_external_tool_config(
     settings: &UserSettings,
     name: &str,
-) -> Result<Option<MergeTool>, ConfigError> {
-    if name == BUILTIN_EDITOR_NAME {
-        return Ok(Some(MergeTool::Builtin));
-    }
-
+) -> Result<Option<ExternalMergeTool>, ConfigError> {
     const TABLE_KEY: &str = "merge-tools";
     let tools_table = settings.config().get_table(TABLE_KEY)?;
     if let Some(v) = tools_table.get(name) {
@@ -203,48 +162,163 @@ pub fn get_tool_config(
         if result.program.is_empty() {
             result.program.clone_from(&name.to_string());
         };
-        Ok(Some(MergeTool::External(result)))
+        Ok(Some(result))
     } else {
         Ok(None)
     }
 }
 
-/// Loads merge tool options from `[merge-tools.<name>]` if `args` is of
-/// unstructured string type.
-pub fn get_tool_config_from_args(
-    settings: &UserSettings,
-    args: &CommandNameAndArgs,
-) -> Result<Option<MergeTool>, ConfigError> {
-    match args {
-        CommandNameAndArgs::String(name) => get_tool_config(settings, name),
-        CommandNameAndArgs::Vec(_) | CommandNameAndArgs::Structured { .. } => Ok(None),
+/// Configured diff editor.
+#[derive(Clone, Debug)]
+pub struct DiffEditor {
+    tool: MergeTool,
+    base_ignores: Arc<GitIgnoreFile>,
+    use_instructions: bool,
+}
+
+impl DiffEditor {
+    /// Creates diff editor of the given name, and loads parameters from the
+    /// settings.
+    pub fn with_name(
+        name: &str,
+        settings: &UserSettings,
+        base_ignores: Arc<GitIgnoreFile>,
+    ) -> Result<Self, MergeToolConfigError> {
+        let tool = get_tool_config(settings, name)?
+            .unwrap_or_else(|| MergeTool::external(ExternalMergeTool::with_program(name)));
+        Self::new_inner(tool, settings, base_ignores)
+    }
+
+    /// Loads the default diff editor from the settings.
+    pub fn from_settings(
+        ui: &Ui,
+        settings: &UserSettings,
+        base_ignores: Arc<GitIgnoreFile>,
+    ) -> Result<Self, MergeToolConfigError> {
+        let args = editor_args_from_settings(ui, settings, "ui.diff-editor")?;
+        let tool = if let CommandNameAndArgs::String(name) = &args {
+            get_tool_config(settings, name)?
+        } else {
+            None
+        }
+        .unwrap_or_else(|| MergeTool::external(ExternalMergeTool::with_edit_args(&args)));
+        Self::new_inner(tool, settings, base_ignores)
+    }
+
+    fn new_inner(
+        tool: MergeTool,
+        settings: &UserSettings,
+        base_ignores: Arc<GitIgnoreFile>,
+    ) -> Result<Self, MergeToolConfigError> {
+        Ok(DiffEditor {
+            tool,
+            base_ignores,
+            use_instructions: settings.config().get_bool("ui.diff-instructions")?,
+        })
+    }
+
+    /// Starts a diff editor on the two directories.
+    pub fn edit(
+        &self,
+        left_tree: &MergedTree,
+        right_tree: &MergedTree,
+        matcher: &dyn Matcher,
+        instructions: Option<&str>,
+    ) -> Result<MergedTreeId, DiffEditError> {
+        match &self.tool {
+            MergeTool::Builtin => {
+                Ok(edit_diff_builtin(left_tree, right_tree, matcher).map_err(Box::new)?)
+            }
+            MergeTool::External(editor) => {
+                let instructions = self.use_instructions.then_some(instructions).flatten();
+                edit_diff_external(
+                    editor,
+                    left_tree,
+                    right_tree,
+                    matcher,
+                    instructions,
+                    self.base_ignores.clone(),
+                )
+            }
+        }
     }
 }
 
-fn get_diff_editor_from_settings(
-    ui: &Ui,
-    settings: &UserSettings,
-) -> Result<MergeTool, ExternalToolError> {
-    let args = editor_args_from_settings(ui, settings, "ui.diff-editor")?;
-    let editor = get_tool_config_from_args(settings, &args)?
-        .unwrap_or_else(|| MergeTool::External(ExternalMergeTool::with_edit_args(&args)));
-    Ok(editor)
+/// Configured 3-way merge editor.
+#[derive(Clone, Debug)]
+pub struct MergeEditor {
+    tool: MergeTool,
 }
 
-fn get_merge_tool_from_settings(
-    ui: &Ui,
-    settings: &UserSettings,
-) -> Result<MergeTool, ExternalToolError> {
-    let args = editor_args_from_settings(ui, settings, "ui.merge-editor")?;
-    let mergetool = get_tool_config_from_args(settings, &args)?
-        .unwrap_or_else(|| MergeTool::External(ExternalMergeTool::with_merge_args(&args)));
-    match mergetool {
-        MergeTool::External(mergetool) if mergetool.merge_args.is_empty() => {
-            Err(ExternalToolError::MergeArgsNotConfigured {
-                tool_name: args.to_string(),
-            })
+impl MergeEditor {
+    /// Creates 3-way merge editor of the given name, and loads parameters from
+    /// the settings.
+    pub fn with_name(name: &str, settings: &UserSettings) -> Result<Self, MergeToolConfigError> {
+        let tool = get_tool_config(settings, name)?
+            .unwrap_or_else(|| MergeTool::external(ExternalMergeTool::with_program(name)));
+        Self::new_inner(name, tool)
+    }
+
+    /// Loads the default 3-way merge editor from the settings.
+    pub fn from_settings(ui: &Ui, settings: &UserSettings) -> Result<Self, MergeToolConfigError> {
+        let args = editor_args_from_settings(ui, settings, "ui.merge-editor")?;
+        let tool = if let CommandNameAndArgs::String(name) = &args {
+            get_tool_config(settings, name)?
+        } else {
+            None
+        }
+        .unwrap_or_else(|| MergeTool::external(ExternalMergeTool::with_merge_args(&args)));
+        Self::new_inner(&args, tool)
+    }
+
+    fn new_inner(name: impl ToString, tool: MergeTool) -> Result<Self, MergeToolConfigError> {
+        if matches!(&tool, MergeTool::External(mergetool) if mergetool.merge_args.is_empty()) {
+            return Err(MergeToolConfigError::MergeArgsNotConfigured {
+                tool_name: name.to_string(),
+            });
+        }
+        Ok(MergeEditor { tool })
+    }
+
+    /// Starts a merge editor for the specified file.
+    pub fn edit_file(
+        &self,
+        tree: &MergedTree,
+        repo_path: &RepoPath,
+    ) -> Result<MergedTreeId, ConflictResolveError> {
+        let conflict = match tree.path_value(repo_path).into_resolved() {
+            Err(conflict) => conflict,
+            Ok(Some(_)) => return Err(ConflictResolveError::NotAConflict(repo_path.to_owned())),
+            Ok(None) => return Err(ConflictResolveError::PathNotFound(repo_path.to_owned())),
+        };
+        let file_merge = conflict.to_file_merge().ok_or_else(|| {
+            let mut summary_bytes: Vec<u8> = vec![];
+            conflict
+                .describe(&mut summary_bytes)
+                .expect("Writing to an in-memory buffer should never fail");
+            ConflictResolveError::NotNormalFiles(
+                repo_path.to_owned(),
+                String::from_utf8_lossy(summary_bytes.as_slice()).to_string(),
+            )
+        })?;
+        // We only support conflicts with 2 sides (3-way conflicts)
+        if file_merge.num_sides() > 2 {
+            return Err(ConflictResolveError::ConflictTooComplicated {
+                path: repo_path.to_owned(),
+                sides: file_merge.num_sides(),
+            });
+        };
+        let content = extract_as_single_hunk(&file_merge, tree.store(), repo_path).block_on();
+
+        match &self.tool {
+            MergeTool::Builtin => {
+                let tree_id = edit_merge_builtin(tree, repo_path, content).map_err(Box::new)?;
+                Ok(tree_id)
+            }
+            MergeTool::External(editor) => external::run_mergetool_external(
+                editor, file_merge, content, repo_path, conflict, tree,
+            ),
         }
-        mergetool => Ok(mergetool),
     }
 }
 
@@ -262,12 +336,69 @@ mod tests {
     }
 
     #[test]
-    fn test_get_diff_editor() {
+    fn test_get_diff_editor_with_name() {
+        let get = |name, config_text| {
+            let config = config_from_string(config_text);
+            let settings = UserSettings::from_config(config);
+            DiffEditor::with_name(name, &settings, GitIgnoreFile::empty()).map(|editor| editor.tool)
+        };
+
+        insta::assert_debug_snapshot!(get(":builtin", "").unwrap(), @"Builtin");
+
+        // Just program name, edit_args are filled by default
+        insta::assert_debug_snapshot!(get("my diff", "").unwrap(), @r###"
+        External(
+            ExternalMergeTool {
+                program: "my diff",
+                diff_args: [
+                    "$left",
+                    "$right",
+                ],
+                edit_args: [
+                    "$left",
+                    "$right",
+                ],
+                merge_args: [],
+                merge_tool_edits_conflict_markers: false,
+            },
+        )
+        "###);
+
+        // Pick from merge-tools
+        insta::assert_debug_snapshot!(get(
+            "foo bar", r#"
+        [merge-tools."foo bar"]
+        edit-args = ["--edit", "args", "$left", "$right"]
+        "#,
+        ).unwrap(), @r###"
+        External(
+            ExternalMergeTool {
+                program: "foo bar",
+                diff_args: [
+                    "$left",
+                    "$right",
+                ],
+                edit_args: [
+                    "--edit",
+                    "args",
+                    "$left",
+                    "$right",
+                ],
+                merge_args: [],
+                merge_tool_edits_conflict_markers: false,
+            },
+        )
+        "###);
+    }
+
+    #[test]
+    fn test_get_diff_editor_from_settings() {
         let get = |text| {
             let config = config_from_string(text);
             let ui = Ui::with_config(&config).unwrap();
             let settings = UserSettings::from_config(config);
-            get_diff_editor_from_settings(&ui, &settings)
+            DiffEditor::from_settings(&ui, &settings, GitIgnoreFile::empty())
+                .map(|editor| editor.tool)
         };
 
         // Default
@@ -411,12 +542,59 @@ mod tests {
     }
 
     #[test]
-    fn test_get_merge_tool() {
+    fn test_get_merge_editor_with_name() {
+        let get = |name, config_text| {
+            let config = config_from_string(config_text);
+            let settings = UserSettings::from_config(config);
+            MergeEditor::with_name(name, &settings).map(|editor| editor.tool)
+        };
+
+        insta::assert_debug_snapshot!(get(":builtin", "").unwrap(), @"Builtin");
+
+        // Just program name
+        insta::assert_debug_snapshot!(get("my diff", "").unwrap_err(), @r###"
+        MergeArgsNotConfigured {
+            tool_name: "my diff",
+        }
+        "###);
+
+        // Pick from merge-tools
+        insta::assert_debug_snapshot!(get(
+            "foo bar", r#"
+        [merge-tools."foo bar"]
+        merge-args = ["$base", "$left", "$right", "$output"]
+        "#,
+        ).unwrap(), @r###"
+        External(
+            ExternalMergeTool {
+                program: "foo bar",
+                diff_args: [
+                    "$left",
+                    "$right",
+                ],
+                edit_args: [
+                    "$left",
+                    "$right",
+                ],
+                merge_args: [
+                    "$base",
+                    "$left",
+                    "$right",
+                    "$output",
+                ],
+                merge_tool_edits_conflict_markers: false,
+            },
+        )
+        "###);
+    }
+
+    #[test]
+    fn test_get_merge_editor_from_settings() {
         let get = |text| {
             let config = config_from_string(text);
             let ui = Ui::with_config(&config).unwrap();
             let settings = UserSettings::from_config(config);
-            get_merge_tool_from_settings(&ui, &settings)
+            MergeEditor::from_settings(&ui, &settings).map(|editor| editor.tool)
         };
 
         // Default
