@@ -169,18 +169,10 @@ where
     L: TemplateProperty<Output = Vec<String>>,
 {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
-        let labels = match self.labels.extract() {
-            Ok(labels) => labels,
-            Err(err) => return formatter.handle_error(err),
-        };
-        for label in &labels {
-            formatter.push_label(label)?;
+        match self.labels.extract() {
+            Ok(labels) => format_labeled(formatter, &self.content, &labels),
+            Err(err) => formatter.handle_error(err),
         }
-        self.content.format(formatter)?;
-        for _label in &labels {
-            formatter.pop_label()?;
-        }
-        Ok(())
     }
 }
 
@@ -632,6 +624,7 @@ impl<O: Clone> TemplateProperty for PropertyPlaceholder<O> {
 pub struct TemplateRenderer<'a, C> {
     template: Box<dyn Template + 'a>,
     placeholder: PropertyPlaceholder<C>,
+    labels: Vec<String>,
 }
 
 impl<'a, C: Clone> TemplateRenderer<'a, C> {
@@ -639,13 +632,25 @@ impl<'a, C: Clone> TemplateRenderer<'a, C> {
         TemplateRenderer {
             template,
             placeholder,
+            labels: Vec::new(),
         }
     }
 
+    /// Returns renderer that will format template with the given `label`.
+    ///
+    /// This is equivalent to wrapping the content template with `label()`
+    /// function. For example, `content.labeled("foo").labeled("bar")` can be
+    /// expressed as `label("bar", label("foo", content))` in template.
+    pub fn labeled(mut self, label: impl Into<String>) -> Self {
+        self.labels.insert(0, label.into());
+        self
+    }
+
     pub fn format(&self, context: &C, formatter: &mut dyn Formatter) -> io::Result<()> {
         let mut wrapper = TemplateFormatter::new(formatter, format_property_error_inline);
-        self.placeholder
-            .with_value(context.clone(), || self.template.format(&mut wrapper))
+        self.placeholder.with_value(context.clone(), || {
+            format_labeled(&mut wrapper, &self.template, &self.labels)
+        })
     }
 }
 
@@ -748,6 +753,21 @@ where
     Ok(())
 }
 
+fn format_labeled<T: Template + ?Sized>(
+    formatter: &mut TemplateFormatter,
+    content: &T,
+    labels: &[String],
+) -> io::Result<()> {
+    for label in labels {
+        formatter.push_label(label)?;
+    }
+    content.format(formatter)?;
+    for _label in labels {
+        formatter.pop_label()?;
+    }
+    Ok(())
+}
+
 type PropertyErrorHandler = fn(&mut dyn Formatter, TemplatePropertyError) -> io::Result<()>;
 
 /// Prints property evaluation error as inline template output.
