@@ -26,17 +26,18 @@ fn test_log_parents() {
     test_env.jj_cmd_ok(&repo_path, &["new", "@-"]);
     test_env.jj_cmd_ok(&repo_path, &["new", "@", "@-"]);
 
-    let template = r#"commit_id ++ "\nP: " ++ parents.map(|c| c.commit_id()) ++ "\n""#;
+    let template =
+        r#"commit_id ++ "\nP: " ++ parents.len() ++ " " ++ parents.map(|c| c.commit_id()) ++ "\n""#;
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", template]);
     insta::assert_snapshot!(stdout, @r###"
     @    c067170d4ca1bc6162b64f7550617ec809647f84
-    ├─╮  P: 4db490c88528133d579540b6900b8098f0c17701 230dd059e1b059aefc0da06a2e5a7dbf22362f22
+    ├─╮  P: 2 4db490c88528133d579540b6900b8098f0c17701 230dd059e1b059aefc0da06a2e5a7dbf22362f22
     ◉ │  4db490c88528133d579540b6900b8098f0c17701
-    ├─╯  P: 230dd059e1b059aefc0da06a2e5a7dbf22362f22
+    ├─╯  P: 1 230dd059e1b059aefc0da06a2e5a7dbf22362f22
     ◉  230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    │  P: 0000000000000000000000000000000000000000
+    │  P: 1 0000000000000000000000000000000000000000
     ◉  0000000000000000000000000000000000000000
-       P:
+       P: 0
     "###);
 
     let template = r#"parents.map(|c| c.commit_id().shortest(4))"#;
@@ -53,19 +54,21 @@ fn test_log_parents() {
     // Commit object isn't printable
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-T", "parents"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse template:  --> 1:1
+    Error: Failed to parse template: Expected expression of type "Template", but actual type is "List<Commit>"
+    Caused by:  --> 1:1
       |
     1 | parents
       | ^-----^
       |
-      = Expected expression of type "Template"
+      = Expected expression of type "Template", but actual type is "List<Commit>"
     "###);
 
     // Redundant argument passed to keyword method
     let template = r#"parents.map(|c| c.commit_id(""))"#;
     let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-T", template]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Failed to parse template:  --> 1:29
+    Error: Failed to parse template: Function "commit_id": Expected 0 arguments
+    Caused by:  --> 1:29
       |
     1 | parents.map(|c| c.commit_id(""))
       |                             ^^
@@ -122,6 +125,56 @@ fn test_log_author_timestamp_utc() {
     "###);
 }
 
+#[cfg(unix)]
+#[test]
+fn test_log_author_timestamp_local() {
+    let mut test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    test_env.add_env_var("TZ", "UTC-05:30");
+    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "author.timestamp().local()"]);
+    insta::assert_snapshot!(stdout, @r###"
+    @  2001-02-03 08:05:07.000 +11:00
+    ◉  1970-01-01 11:00:00.000 +11:00
+    "###);
+    test_env.add_env_var("TZ", "UTC+10:00");
+    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", "author.timestamp().local()"]);
+    insta::assert_snapshot!(stdout, @r###"
+    @  2001-02-03 08:05:07.000 +11:00
+    ◉  1970-01-01 11:00:00.000 +11:00
+    "###);
+}
+
+#[test]
+fn test_mine_is_true_when_author_is_user() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(
+        &repo_path,
+        &[
+            "--config-toml=user.email='johndoe@example.com'",
+            "--config-toml=user.name='John Doe'",
+            "new",
+        ],
+    );
+
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "log",
+            "-T",
+            r#"coalesce(if(mine, "mine"), author.email(), email_placeholder)"#,
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    @  johndoe@example.com
+    ◉  mine
+    ◉  (no email set)
+    "###);
+}
+
 #[test]
 fn test_log_default() {
     let test_env = TestEnvironment::default();
@@ -136,9 +189,9 @@ fn test_log_default() {
     // Test default log output format
     let stdout = test_env.jj_cmd_success(&repo_path, &["log"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  kkmpptxz test.user@example.com 2001-02-03 04:05:09.000 +07:00 my-branch 9de54178
+    @  kkmpptxz test.user@example.com 2001-02-03 08:05:09 my-branch 9de54178
     │  (empty) description 1
-    ◉  qpvuntsm test.user@example.com 2001-02-03 04:05:08.000 +07:00 4291e264
+    ◉  qpvuntsm test.user@example.com 2001-02-03 08:05:08 4291e264
     │  add a file
     ◉  zzzzzzzz root() 00000000
     "###);
@@ -146,9 +199,9 @@ fn test_log_default() {
     // Color
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "--color=always"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  [1m[38;5;13mk[38;5;8mkmpptxz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 04:05:09.000 +07:00[39m [38;5;13mmy-branch[39m [38;5;12m9[38;5;8mde54178[39m[0m
+    @  [1m[38;5;13mk[38;5;8mkmpptxz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:09[39m [38;5;13mmy-branch[39m [38;5;12m9[38;5;8mde54178[39m[0m
     │  [1m[38;5;10m(empty)[39m description 1[0m
-    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:08.000 +07:00[39m [1m[38;5;4m4[0m[38;5;8m291e264[39m
+    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:08[39m [1m[38;5;4m4[0m[38;5;8m291e264[39m
     │  add a file
     ◉  [1m[38;5;5mz[0m[38;5;8mzzzzzzz[39m [38;5;2mroot()[39m [1m[38;5;4m0[0m[38;5;8m0000000[39m
     "###);
@@ -156,9 +209,9 @@ fn test_log_default() {
     // Color without graph
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "--color=always", "--no-graph"]);
     insta::assert_snapshot!(stdout, @r###"
-    [1m[38;5;13mk[38;5;8mkmpptxz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 04:05:09.000 +07:00[39m [38;5;13mmy-branch[39m [38;5;12m9[38;5;8mde54178[39m[0m
+    [1m[38;5;13mk[38;5;8mkmpptxz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:09[39m [38;5;13mmy-branch[39m [38;5;12m9[38;5;8mde54178[39m[0m
     [1m[38;5;10m(empty)[39m description 1[0m
-    [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:08.000 +07:00[39m [1m[38;5;4m4[0m[38;5;8m291e264[39m
+    [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:08[39m [1m[38;5;4m4[0m[38;5;8m291e264[39m
     add a file
     [1m[38;5;5mz[0m[38;5;8mzzzzzzz[39m [38;5;2mroot()[39m [1m[38;5;4m0[0m[38;5;8m0000000[39m
     "###);
@@ -169,7 +222,10 @@ fn test_log_builtin_templates() {
     let test_env = TestEnvironment::default();
     test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
     let repo_path = test_env.env_root().join("repo");
-    let render = |template| test_env.jj_cmd_success(&repo_path, &["log", "-T", template]);
+    // Render without graph and append "[EOF]" marker to test line ending
+    let render = |template| {
+        test_env.jj_cmd_success(&repo_path, &["log", "-T", template, "--no-graph"]) + "[EOF]\n"
+    };
 
     test_env.jj_cmd_ok(
         &repo_path,
@@ -182,52 +238,57 @@ fn test_log_builtin_templates() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "my-branch"]);
 
     insta::assert_snapshot!(render(r#"builtin_log_oneline"#), @r###"
-    @  rlvkpnrz (no email set) 2001-02-03 04:05:08.000 +07:00 my-branch dc315397 (empty) (no description set)
-    ◉  qpvuntsm test.user 2001-02-03 04:05:07.000 +07:00 230dd059 (empty) (no description set)
-    ◉  zzzzzzzz root() 00000000
+    rlvkpnrz (no email set) 2001-02-03 08:05:08 my-branch dc315397 (empty) (no description set)
+    qpvuntsm test.user 2001-02-03 08:05:07 230dd059 (empty) (no description set)
+    zzzzzzzz root() 00000000
+    [EOF]
     "###);
 
     insta::assert_snapshot!(render(r#"builtin_log_compact"#), @r###"
-    @  rlvkpnrz (no email set) 2001-02-03 04:05:08.000 +07:00 my-branch dc315397
-    │  (empty) (no description set)
-    ◉  qpvuntsm test.user@example.com 2001-02-03 04:05:07.000 +07:00 230dd059
-    │  (empty) (no description set)
-    ◉  zzzzzzzz root() 00000000
+    rlvkpnrz (no email set) 2001-02-03 08:05:08 my-branch dc315397
+    (empty) (no description set)
+    qpvuntsm test.user@example.com 2001-02-03 08:05:07 230dd059
+    (empty) (no description set)
+    zzzzzzzz root() 00000000
+    [EOF]
     "###);
 
     insta::assert_snapshot!(render(r#"builtin_log_comfortable"#), @r###"
-    @  rlvkpnrz (no email set) 2001-02-03 04:05:08.000 +07:00 my-branch dc315397
-    │  (empty) (no description set)
-    │
-    ◉  qpvuntsm test.user@example.com 2001-02-03 04:05:07.000 +07:00 230dd059
-    │  (empty) (no description set)
-    │
-    ◉  zzzzzzzz root() 00000000
+    rlvkpnrz (no email set) 2001-02-03 08:05:08 my-branch dc315397
+    (empty) (no description set)
+
+    qpvuntsm test.user@example.com 2001-02-03 08:05:07 230dd059
+    (empty) (no description set)
+
+    zzzzzzzz root() 00000000
+
+    [EOF]
     "###);
 
     insta::assert_snapshot!(render(r#"builtin_log_detailed"#), @r###"
-    @  Commit ID: dc31539712c7294d1d712cec63cef4504b94ca74
-    │  Change ID: rlvkpnrzqnoowoytxnquwvuryrwnrmlp
-    │  Branches: my-branch
-    │  Author: (no name set) <(no email set)> (2001-02-03 04:05:08.000 +07:00)
-    │  Committer: (no name set) <(no email set)> (2001-02-03 04:05:08.000 +07:00)
-    │
-    │      (no description set)
-    │
-    ◉  Commit ID: 230dd059e1b059aefc0da06a2e5a7dbf22362f22
-    │  Change ID: qpvuntsmwlqtpsluzzsnyyzlmlwvmlnu
-    │  Author: Test User <test.user@example.com> (2001-02-03 04:05:07.000 +07:00)
-    │  Committer: Test User <test.user@example.com> (2001-02-03 04:05:07.000 +07:00)
-    │
-    │      (no description set)
-    │
-    ◉  Commit ID: 0000000000000000000000000000000000000000
-       Change ID: zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
-       Author: (no name set) <(no email set)> (1970-01-01 00:00:00.000 +00:00)
-       Committer: (no name set) <(no email set)> (1970-01-01 00:00:00.000 +00:00)
+    Commit ID: dc31539712c7294d1d712cec63cef4504b94ca74
+    Change ID: rlvkpnrzqnoowoytxnquwvuryrwnrmlp
+    Branches: my-branch
+    Author: (no name set) <(no email set)> (2001-02-03 08:05:08)
+    Committer: (no name set) <(no email set)> (2001-02-03 08:05:08)
 
-           (no description set)
+        (no description set)
 
+    Commit ID: 230dd059e1b059aefc0da06a2e5a7dbf22362f22
+    Change ID: qpvuntsmwlqtpsluzzsnyyzlmlwvmlnu
+    Author: Test User <test.user@example.com> (2001-02-03 08:05:07)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:07)
+
+        (no description set)
+
+    Commit ID: 0000000000000000000000000000000000000000
+    Change ID: zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
+    Author: (no name set) <(no email set)> (1970-01-01 11:00:00)
+    Committer: (no name set) <(no email set)> (1970-01-01 11:00:00)
+
+        (no description set)
+
+    [EOF]
     "###);
 }
 
@@ -250,24 +311,24 @@ fn test_log_builtin_templates_colored() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "my-branch"]);
 
     insta::assert_snapshot!(render(r#"builtin_log_oneline"#), @r###"
-    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;9m(no email set)[39m [38;5;14m2001-02-03 04:05:08.000 +07:00[39m [38;5;13mmy-branch[39m [38;5;12md[38;5;8mc315397[39m [38;5;10m(empty)[39m [38;5;10m(no description set)[39m[0m
-    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user[39m [38;5;6m2001-02-03 04:05:07.000 +07:00[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m [38;5;2m(empty)[39m [38;5;2m(no description set)[39m
+    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;9m(no email set)[39m [38;5;14m2001-02-03 08:05:08[39m [38;5;13mmy-branch[39m [38;5;12md[38;5;8mc315397[39m [38;5;10m(empty)[39m [38;5;10m(no description set)[39m[0m
+    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user[39m [38;5;6m2001-02-03 08:05:07[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m [38;5;2m(empty)[39m [38;5;2m(no description set)[39m
     ◉  [1m[38;5;5mz[0m[38;5;8mzzzzzzz[39m [38;5;2mroot()[39m [1m[38;5;4m0[0m[38;5;8m0000000[39m
     "###);
 
     insta::assert_snapshot!(render(r#"builtin_log_compact"#), @r###"
-    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;9m(no email set)[39m [38;5;14m2001-02-03 04:05:08.000 +07:00[39m [38;5;13mmy-branch[39m [38;5;12md[38;5;8mc315397[39m[0m
+    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;9m(no email set)[39m [38;5;14m2001-02-03 08:05:08[39m [38;5;13mmy-branch[39m [38;5;12md[38;5;8mc315397[39m[0m
     │  [1m[38;5;10m(empty)[39m [38;5;10m(no description set)[39m[0m
-    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:07.000 +07:00[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
+    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:07[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
     │  [38;5;2m(empty)[39m [38;5;2m(no description set)[39m
     ◉  [1m[38;5;5mz[0m[38;5;8mzzzzzzz[39m [38;5;2mroot()[39m [1m[38;5;4m0[0m[38;5;8m0000000[39m
     "###);
 
     insta::assert_snapshot!(render(r#"builtin_log_comfortable"#), @r###"
-    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;9m(no email set)[39m [38;5;14m2001-02-03 04:05:08.000 +07:00[39m [38;5;13mmy-branch[39m [38;5;12md[38;5;8mc315397[39m[0m
+    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;9m(no email set)[39m [38;5;14m2001-02-03 08:05:08[39m [38;5;13mmy-branch[39m [38;5;12md[38;5;8mc315397[39m[0m
     │  [1m[38;5;10m(empty)[39m [38;5;10m(no description set)[39m[0m
     │
-    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:07.000 +07:00[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
+    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:07[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
     │  [38;5;2m(empty)[39m [38;5;2m(no description set)[39m
     │
     ◉  [1m[38;5;5mz[0m[38;5;8mzzzzzzz[39m [38;5;2mroot()[39m [1m[38;5;4m0[0m[38;5;8m0000000[39m
@@ -277,22 +338,22 @@ fn test_log_builtin_templates_colored() {
     @  Commit ID: [38;5;4mdc31539712c7294d1d712cec63cef4504b94ca74[39m
     │  Change ID: [38;5;5mrlvkpnrzqnoowoytxnquwvuryrwnrmlp[39m
     │  Branches: [38;5;5mmy-branch[39m
-    │  Author: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m2001-02-03 04:05:08.000 +07:00[39m)
-    │  Committer: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m2001-02-03 04:05:08.000 +07:00[39m)
+    │  Author: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m2001-02-03 08:05:08[39m)
+    │  Committer: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m2001-02-03 08:05:08[39m)
     │
     │  [38;5;2m    (no description set)[39m
     │
     ◉  Commit ID: [38;5;4m230dd059e1b059aefc0da06a2e5a7dbf22362f22[39m
     │  Change ID: [38;5;5mqpvuntsmwlqtpsluzzsnyyzlmlwvmlnu[39m
-    │  Author: Test User <[38;5;3mtest.user@example.com[39m> ([38;5;6m2001-02-03 04:05:07.000 +07:00[39m)
-    │  Committer: Test User <[38;5;3mtest.user@example.com[39m> ([38;5;6m2001-02-03 04:05:07.000 +07:00[39m)
+    │  Author: Test User <[38;5;3mtest.user@example.com[39m> ([38;5;6m2001-02-03 08:05:07[39m)
+    │  Committer: Test User <[38;5;3mtest.user@example.com[39m> ([38;5;6m2001-02-03 08:05:07[39m)
     │
     │  [38;5;2m    (no description set)[39m
     │
     ◉  Commit ID: [38;5;4m0000000000000000000000000000000000000000[39m
        Change ID: [38;5;5mzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz[39m
-       Author: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m1970-01-01 00:00:00.000 +00:00[39m)
-       Committer: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m1970-01-01 00:00:00.000 +00:00[39m)
+       Author: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m1970-01-01 11:00:00[39m)
+       Committer: [38;5;1m(no name set)[39m <[38;5;1m(no email set)[39m> ([38;5;6m1970-01-01 11:00:00[39m)
 
        [38;5;2m    (no description set)[39m
 
@@ -310,7 +371,7 @@ fn test_log_obslog_divergence() {
     let stdout = test_env.jj_cmd_success(&repo_path, &["log"]);
     // No divergence
     insta::assert_snapshot!(stdout, @r###"
-    @  qpvuntsm test.user@example.com 2001-02-03 04:05:08.000 +07:00 7a17d52e
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:08 7a17d52e
     │  description 1
     ◉  zzzzzzzz root() 00000000
     "###);
@@ -322,9 +383,9 @@ fn test_log_obslog_divergence() {
     );
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["log"]);
     insta::assert_snapshot!(stdout, @r###"
-    ◉  qpvuntsm?? test.user@example.com 2001-02-03 04:05:10.000 +07:00 8979953d
+    ◉  qpvuntsm?? test.user@example.com 2001-02-03 08:05:10 8979953d
     │  description 2
-    │ @  qpvuntsm?? test.user@example.com 2001-02-03 04:05:08.000 +07:00 7a17d52e
+    │ @  qpvuntsm?? test.user@example.com 2001-02-03 08:05:08 7a17d52e
     ├─╯  description 1
     ◉  zzzzzzzz root() 00000000
     "###);
@@ -335,9 +396,9 @@ fn test_log_obslog_divergence() {
     // Color
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "--color=always"]);
     insta::assert_snapshot!(stdout, @r###"
-    ◉  [1m[4m[38;5;1mq[0m[38;5;1mpvuntsm??[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:10.000 +07:00[39m [1m[38;5;4m8[0m[38;5;8m979953d[39m
+    ◉  [1m[4m[38;5;1mq[0m[38;5;1mpvuntsm??[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:10[39m [1m[38;5;4m8[0m[38;5;8m979953d[39m
     │  description 2
-    │ @  [1m[4m[38;5;1mq[24mpvuntsm[38;5;9m??[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 04:05:08.000 +07:00[39m [38;5;12m7[38;5;8ma17d52e[39m[0m
+    │ @  [1m[4m[38;5;1mq[24mpvuntsm[38;5;9m??[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:08[39m [38;5;12m7[38;5;8ma17d52e[39m[0m
     ├─╯  [1mdescription 1[0m
     ◉  [1m[38;5;5mz[0m[38;5;8mzzzzzzz[39m [38;5;2mroot()[39m [1m[38;5;4m0[0m[38;5;8m0000000[39m
     "###);
@@ -345,22 +406,22 @@ fn test_log_obslog_divergence() {
     // Obslog and hidden divergent
     let stdout = test_env.jj_cmd_success(&repo_path, &["obslog"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  qpvuntsm?? test.user@example.com 2001-02-03 04:05:08.000 +07:00 7a17d52e
+    @  qpvuntsm?? test.user@example.com 2001-02-03 08:05:08 7a17d52e
     │  description 1
-    ◉  qpvuntsm hidden test.user@example.com 2001-02-03 04:05:08.000 +07:00 3b68ce25
+    ◉  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:08 3b68ce25
     │  (no description set)
-    ◉  qpvuntsm hidden test.user@example.com 2001-02-03 04:05:07.000 +07:00 230dd059
+    ◉  qpvuntsm hidden test.user@example.com 2001-02-03 08:05:07 230dd059
        (empty) (no description set)
     "###);
 
     // Colored obslog
     let stdout = test_env.jj_cmd_success(&repo_path, &["obslog", "--color=always"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  [1m[4m[38;5;1mq[24mpvuntsm[38;5;9m??[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 04:05:08.000 +07:00[39m [38;5;12m7[38;5;8ma17d52e[39m[0m
+    @  [1m[4m[38;5;1mq[24mpvuntsm[38;5;9m??[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:08[39m [38;5;12m7[38;5;8ma17d52e[39m[0m
     │  [1mdescription 1[0m
-    ◉  [1m[39mq[0m[38;5;8mpvuntsm[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:08.000 +07:00[39m [1m[38;5;4m3[0m[38;5;8mb68ce25[39m
+    ◉  [1m[39mq[0m[38;5;8mpvuntsm[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:08[39m [1m[38;5;4m3[0m[38;5;8mb68ce25[39m
     │  [38;5;3m(no description set)[39m
-    ◉  [1m[39mq[0m[38;5;8mpvuntsm[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:07.000 +07:00[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
+    ◉  [1m[39mq[0m[38;5;8mpvuntsm[39m hidden [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:07[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
        [38;5;2m(empty)[39m [38;5;2m(no description set)[39m
     "###);
 }
@@ -464,11 +525,25 @@ fn test_log_git_head() {
 
     test_env.jj_cmd_ok(&repo_path, &["new", "-m=initial"]);
     std::fs::write(repo_path.join("file"), "foo\n").unwrap();
+
+    let template = r#"
+    separate(", ",
+      if(git_head, "name: " ++ git_head.name()),
+      "remote: " ++ git_head.remote(),
+    ) ++ "\n"
+    "#;
+    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-T", template]);
+    insta::assert_snapshot!(stdout, @r###"
+    @  remote: <Error: No RefName available>
+    ◉  name: HEAD, remote: git
+    ◉  remote: <Error: No RefName available>
+    "###);
+
     let stdout = test_env.jj_cmd_success(&repo_path, &["log", "--color=always"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 04:05:09.000 +07:00[39m [38;5;12m5[38;5;8m0aaf475[39m[0m
+    @  [1m[38;5;13mr[38;5;8mlvkpnrz[39m [38;5;3mtest.user@example.com[39m [38;5;14m2001-02-03 08:05:09[39m [38;5;12m5[38;5;8m0aaf475[39m[0m
     │  [1minitial[0m
-    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 04:05:07.000 +07:00[39m [38;5;2mHEAD@git[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
+    ◉  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [38;5;3mtest.user@example.com[39m [38;5;6m2001-02-03 08:05:07[39m [38;5;2mHEAD@git[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m
     │  [38;5;2m(empty)[39m [38;5;2m(no description set)[39m
     ◉  [1m[38;5;5mz[0m[38;5;8mzzzzzzz[39m [38;5;2mroot()[39m [1m[38;5;4m0[0m[38;5;8m0000000[39m
     "###);
@@ -493,7 +568,7 @@ fn test_log_customize_short_id() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    @  Q_pvun test.user@example.com 2001-02-03 04:05:08.000 +07:00 6_9542
+    @  Q_pvun test.user@example.com 2001-02-03 08:05:08 6_9542
     │  (empty) first
     ◉  Z_zzzz root() 0_0000
     "###);
@@ -511,8 +586,173 @@ fn test_log_customize_short_id() {
         ],
     );
     insta::assert_snapshot!(stdout, @r###"
-    @  QPVUNTSM test.user@example.com 2001-02-03 04:05:08.000 +07:00 69542c19
+    @  QPVUNTSM test.user@example.com 2001-02-03 08:05:08 69542c19
     │  (empty) first
     ◉  ZZZZZZZZ root() 00000000
     "###);
 }
+
+#[test]
+fn test_log_immutable() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mA", "root()"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mB"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mC"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mD", "root()"]);
+
+    let template = r#"
+    separate(" ",
+      description.first_line(),
+      branches,
+      if(immutable, "[immutable]"),
+    ) ++ "\n"
+    "#;
+
+    test_env.add_config("revset-aliases.'immutable_heads()' = 'main'");
+    let stdout = test_env.jj_cmd_success(&repo_path, &["log", "-r::", "-T", template]);
+    insta::assert_snapshot!(stdout, @r###"
+    @  D
+    │ ◉  C
+    │ ◉  B main [immutable]
+    │ ◉  A [immutable]
+    ├─╯
+    ◉  [immutable]
+    "###);
+
+    // Suppress error that could be detected earlier
+    test_env.add_config("revsets.short-prefixes = ''");
+
+    test_env.add_config("revset-aliases.'immutable_heads()' = 'unknown_fn()'");
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r::", "-T", template]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Failed to parse template: Failed to parse revset
+    Caused by:
+    1:  --> 5:10
+      |
+    5 |       if(immutable, "[immutable]"),
+      |          ^-------^
+      |
+      = Failed to parse revset
+    2:  --> 1:1
+      |
+    1 | unknown_fn()
+      | ^--------^
+      |
+      = Function "unknown_fn" doesn't exist
+    "###);
+
+    test_env.add_config("revset-aliases.'immutable_heads()' = 'unknown_symbol'");
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["log", "-r::", "-T", template]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Failed to parse template: Failed to evaluate revset
+    Caused by:
+    1:  --> 5:10
+      |
+    5 |       if(immutable, "[immutable]"),
+      |          ^-------^
+      |
+      = Failed to evaluate revset
+    2: Revision "unknown_symbol" doesn't exist
+    "###);
+}
+
+#[test]
+fn test_log_contained_in() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["init", "repo", "--git"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mA", "root()"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mB"]);
+    test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mC"]);
+    test_env.jj_cmd_ok(&repo_path, &["new", "-mD", "root()"]);
+
+    let template_for_revset = |revset: &str| {
+        format!(
+            r#"
+    separate(" ",
+      description.first_line(),
+      branches,
+      if(self.contained_in("{revset}"), "[contained_in]"),
+    ) ++ "\n"
+    "#
+        )
+    };
+
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "log",
+            "-r::",
+            "-T",
+            &template_for_revset(r#"description(A)::"#),
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    @  D
+    │ ◉  C [contained_in]
+    │ ◉  B main [contained_in]
+    │ ◉  A [contained_in]
+    ├─╯
+    ◉
+    "###);
+
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "log",
+            "-r::",
+            "-T",
+            &template_for_revset(r#"visible_heads()"#),
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    @  D [contained_in]
+    │ ◉  C [contained_in]
+    │ ◉  B main
+    │ ◉  A
+    ├─╯
+    ◉
+    "###);
+
+    // Suppress error that could be detected earlier
+    let stderr = test_env.jj_cmd_failure(
+        &repo_path,
+        &["log", "-r::", "-T", &template_for_revset("unknown_fn()")],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Failed to parse template: Failed to parse revset
+    Caused by:
+    1:  --> 5:28
+      |
+    5 |       if(self.contained_in("unknown_fn()"), "[contained_in]"),
+      |                            ^------------^
+      |
+      = Failed to parse revset
+    2:  --> 1:1
+      |
+    1 | unknown_fn()
+      | ^--------^
+      |
+      = Function "unknown_fn" doesn't exist
+    "###);
+
+    let stderr = test_env.jj_cmd_failure(
+        &repo_path,
+        &["log", "-r::", "-T", &template_for_revset("unknown_symbol")],
+    );
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Failed to parse template: Failed to evaluate revset
+    Caused by:
+    1:  --> 5:28
+      |
+    5 |       if(self.contained_in("unknown_symbol"), "[contained_in]"),
+      |                            ^--------------^
+      |
+      = Failed to evaluate revset
+    2: Revision "unknown_symbol" doesn't exist
+    "###);
+}
