@@ -28,11 +28,11 @@ fn test_rewrite_immutable_generic() {
     std::fs::write(repo_path.join("file"), "c").unwrap();
     let stdout = test_env.jj_cmd_success(&repo_path, &["log"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  mzvwutvl test.user@example.com 2001-02-03 04:05:12.000 +07:00 78ebd449
+    @  mzvwutvl test.user@example.com 2001-02-03 08:05:12 78ebd449
     │  c
-    │ ◉  kkmpptxz test.user@example.com 2001-02-03 04:05:10.000 +07:00 main c8d4c7ca
+    │ ◉  kkmpptxz test.user@example.com 2001-02-03 08:05:10 main c8d4c7ca
     ├─╯  b
-    ◉  qpvuntsm test.user@example.com 2001-02-03 04:05:08.000 +07:00 46a8dc51
+    ◉  qpvuntsm test.user@example.com 2001-02-03 08:05:08 46a8dc51
     │  a
     ◉  zzzzzzzz root() 00000000
     "###);
@@ -42,13 +42,13 @@ fn test_rewrite_immutable_generic() {
     let stderr = test_env.jj_cmd_failure(&repo_path, &["edit", "main"]);
     insta::assert_snapshot!(stderr, @r###"
     Error: Commit c8d4c7ca95d0 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // Cannot rewrite an ancestor of the configured set
     let stderr = test_env.jj_cmd_failure(&repo_path, &["edit", "main-"]);
     insta::assert_snapshot!(stderr, @r###"
     Error: Commit 46a8dc5175be is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // Cannot rewrite the root commit even with an empty set of immutable commits
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
@@ -56,25 +56,56 @@ fn test_rewrite_immutable_generic() {
     insta::assert_snapshot!(stderr, @r###"
     Error: The root commit 000000000000 is immutable
     "###);
+
+    // Error mutating the repo if immutable_heads() uses a ref that can't be
+    // resolved
+    test_env.add_config(r#"revset-aliases."immutable_heads()" = "branch_that_does_not_exist""#);
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "main"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Config error: Invalid `revset-aliases.immutable_heads()`
+    Caused by: Revision "branch_that_does_not_exist" doesn't exist
+    For help, see https://github.com/martinvonz/jj/blob/main/docs/config.md.
+    "###);
+
+    // Can use --ignore-immutable to override
+    test_env.add_config(r#"revset-aliases."immutable_heads()" = "main""#);
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["--ignore-immutable", "edit", "main"]);
+    insta::assert_snapshot!(stdout, @r###"
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: kkmpptxz c8d4c7ca main | b
+    Parent commit      : qpvuntsm 46a8dc51 a
+    Added 0 files, modified 1 files, removed 0 files
+    "###);
+    // ... but not the root commit
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["--ignore-immutable", "edit", "root()"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: The root commit 000000000000 is immutable
+    "###);
+
+    // Mutating the repo works if ref is wrapped in present()
+    test_env.add_config(
+        r#"revset-aliases."immutable_heads()" = "present(branch_that_does_not_exist)""#,
+    );
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["new", "main"]);
+    insta::assert_snapshot!(stdout, @r###"
+    "###);
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: wqnwkozp de8b93b4 (empty) (no description set)
+    Parent commit      : kkmpptxz c8d4c7ca main | b
+    "###);
+
     // Error if we redefine immutable_heads() with an argument
-    // TODO: This error comes from the built-in definition of
-    // `revsets.short-prefixes`. That's not clear to the user.
     test_env.add_config(r#"revset-aliases."immutable_heads(foo)" = "none()""#);
     let stderr = test_env.jj_cmd_failure(&repo_path, &["edit", "root()"]);
     insta::assert_snapshot!(stderr, @r###"
-    Config error: Invalid `revsets.short-prefixes`:  --> 1:31
-      |
-    1 | @ | ancestors(immutable_heads().., 2) | heads(immutable_heads())
-      |                               ^
-      |
-      = Invalid arguments to revset function "immutable_heads": Expected 1 arguments
-    For help, see https://github.com/martinvonz/jj/blob/main/docs/config.md.
+    Error: The `revset-aliases.immutable_heads()` function must be declared without arguments
     "###);
     // ... even if we also update the built-in call sites
     test_env.add_config(r#"revsets.short-prefixes = "immutable_heads(root())""#);
     let stderr = test_env.jj_cmd_failure(&repo_path, &["edit", "root()"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: The `revset-aliases.immutable_heads()` function must be declared without arguments.
+    Error: The `revset-aliases.immutable_heads()` function must be declared without arguments
     "###);
 }
 
@@ -93,18 +124,19 @@ fn test_rewrite_immutable_commands() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "main"]);
     test_env.jj_cmd_ok(&repo_path, &["new", "description(b)"]);
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "main""#);
+    test_env.add_config(r#"revset-aliases."trunk()" = "main""#);
 
-    // Log shows mutable commits and immutable heads by default
+    // Log shows mutable commits, their parents, and trunk() by default
     let stdout = test_env.jj_cmd_success(&repo_path, &["log"]);
     insta::assert_snapshot!(stdout, @r###"
-    @  yqosqzyt test.user@example.com 2001-02-03 04:05:13.000 +07:00 3f89addf
+    @  yqosqzyt test.user@example.com 2001-02-03 08:05:13 3f89addf
     │  (empty) (no description set)
-    │ ◉  mzvwutvl test.user@example.com 2001-02-03 04:05:11.000 +07:00 main 16ca9d80 conflict
+    │ ◉  mzvwutvl test.user@example.com 2001-02-03 08:05:11 main 406c181c conflict
     ╭─┤  (empty) merge
     │ │
     │ ~
     │
-    ◉  kkmpptxz test.user@example.com 2001-02-03 04:05:10.000 +07:00 c8d4c7ca
+    ◉  kkmpptxz test.user@example.com 2001-02-03 08:05:10 c8d4c7ca
     │  b
     ~
     "###);
@@ -112,109 +144,131 @@ fn test_rewrite_immutable_commands() {
     // abandon
     let stderr = test_env.jj_cmd_failure(&repo_path, &["abandon", "main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // chmod
     let stderr = test_env.jj_cmd_failure(&repo_path, &["chmod", "-r=main", "x", "file"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // describe
     let stderr = test_env.jj_cmd_failure(&repo_path, &["describe", "main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // diffedit
     let stderr = test_env.jj_cmd_failure(&repo_path, &["diffedit", "-r=main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // edit
     let stderr = test_env.jj_cmd_failure(&repo_path, &["edit", "main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // move --from
     let stderr = test_env.jj_cmd_failure(&repo_path, &["move", "--from=main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Warning: `jj move` is deprecated; use `jj squash` instead, which is equivalent
+    Warning: `jj move` will be removed in a future version, and this will be a hard error
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // move --to
     let stderr = test_env.jj_cmd_failure(&repo_path, &["move", "--to=main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Warning: `jj move` is deprecated; use `jj squash` instead, which is equivalent
+    Warning: `jj move` will be removed in a future version, and this will be a hard error
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // new --insert-before
     let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "--insert-before", "main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // new --insert-after parent_of_main
     let stderr = test_env.jj_cmd_failure(&repo_path, &["new", "--insert-after", "description(b)"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    "###);
+    // parallelize
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["parallelize", "description(b)", "main"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // rebase -s
     let stderr = test_env.jj_cmd_failure(&repo_path, &["rebase", "-s=main", "-d=@"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // rebase -b
     let stderr = test_env.jj_cmd_failure(&repo_path, &["rebase", "-b=main", "-d=@"]);
     insta::assert_snapshot!(stderr, @r###"
     Error: Commit 6e11f430f297 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // rebase -r
     let stderr = test_env.jj_cmd_failure(&repo_path, &["rebase", "-r=main", "-d=@"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // resolve
     let stderr = test_env.jj_cmd_failure(&repo_path, &["resolve", "-r=description(merge)", "file"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // restore -c
     let stderr = test_env.jj_cmd_failure(&repo_path, &["restore", "-c=main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // restore --to
     let stderr = test_env.jj_cmd_failure(&repo_path, &["restore", "--to=main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // split
     let stderr = test_env.jj_cmd_failure(&repo_path, &["split", "-r=main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
-    // squash
-    let stderr = test_env.jj_cmd_failure(&repo_path, &["squash", "-r=main"]);
+    // squash -r
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["squash", "-r=description(b)"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit c8d4c7ca95d0 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    "###);
+    // squash --from
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["squash", "--from=main"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    "###);
+    // squash --into
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["squash", "--into=main"]);
+    insta::assert_snapshot!(stderr, @r###"
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
     // unsquash
     let stderr = test_env.jj_cmd_failure(&repo_path, &["unsquash", "-r=main"]);
     insta::assert_snapshot!(stderr, @r###"
-    Error: Commit 16ca9d800b08 is immutable
-    Hint: Configure the set of immutable commits via `revset-aliases.immutable_heads()`.
+    Error: Commit 406c181c04d8 is immutable
+    Hint: Pass `--ignore-immutable` or configure the set of immutable commits via `revset-aliases.immutable_heads()`.
     "###);
 }
