@@ -17,20 +17,24 @@
 use std::cmp::{max, min, Ordering};
 use std::collections::{BTreeSet, BinaryHeap, HashSet};
 use std::iter;
-use std::sync::Arc;
+use std::sync::{Arc, Mutex};
 
 use itertools::Itertools;
+use ref_cast::{ref_cast_custom, RefCastCustom};
 
 use super::entry::{
     IndexEntry, IndexPosition, IndexPositionByGeneration, LocalPosition, SmallIndexPositionsVec,
+    SmallLocalPositionsVec,
 };
 use super::readonly::ReadonlyIndexSegment;
-use super::rev_walk::RevWalk;
-use crate::backend::{ChangeId, CommitId, ObjectId};
-use crate::index::{HexPrefix, Index, PrefixResolution};
+use super::rev_walk::AncestorsBitSet;
+use super::revset_engine;
+use crate::backend::{ChangeId, CommitId};
+use crate::hex_util;
+use crate::index::{AllHeadsForGcUnsupported, ChangeIdIndex, Index};
+use crate::object_id::{HexPrefix, ObjectId, PrefixResolution};
 use crate::revset::{ResolvedExpression, Revset, RevsetEvaluationError};
 use crate::store::Store;
-use crate::{backend, default_revset_engine};
 
 pub(super) trait IndexSegment: Send + Sync {
     fn num_parent_commits(&self) -> u32;
@@ -41,7 +45,7 @@ pub(super) trait IndexSegment: Send + Sync {
 
     fn name(&self) -> Option<String>;
 
-    fn commit_id_to_pos(&self, commit_id: &CommitId) -> Option<IndexPosition>;
+    fn commit_id_to_pos(&self, commit_id: &CommitId) -> Option<LocalPosition>;
 
     /// Suppose the given `commit_id` exists, returns the previous and next
     /// commit ids in lexicographical order.
@@ -52,6 +56,16 @@ pub(super) trait IndexSegment: Send + Sync {
 
     fn resolve_commit_id_prefix(&self, prefix: &HexPrefix) -> PrefixResolution<CommitId>;
 
+    fn resolve_neighbor_change_ids(
+        &self,
+        change_id: &ChangeId,
+    ) -> (Option<ChangeId>, Option<ChangeId>);
+
+    fn resolve_change_id_prefix(
+        &self,
+        prefix: &HexPrefix,
+    ) -> PrefixResolution<(ChangeId, SmallLocalPositionsVec)>;
+
     fn generation_number(&self, local_pos: LocalPosition) -> u32;
 
     fn commit_id(&self, local_pos: LocalPosition) -> CommitId;
@@ -63,47 +77,55 @@ pub(super) trait IndexSegment: Send + Sync {
     fn parent_positions(&self, local_pos: LocalPosition) -> SmallIndexPositionsVec;
 }
 
+pub(super) type DynIndexSegment = dyn IndexSegment;
+
 /// Abstraction over owned and borrowed types that can be cheaply converted to
 /// a `CompositeIndex` reference.
 pub trait AsCompositeIndex {
     /// Returns reference wrapper that provides global access to this index.
-    fn as_composite(&self) -> CompositeIndex<'_>;
+    fn as_composite(&self) -> &CompositeIndex;
 }
 
 impl<T: AsCompositeIndex + ?Sized> AsCompositeIndex for &T {
-    fn as_composite(&self) -> CompositeIndex<'_> {
+    fn as_composite(&self) -> &CompositeIndex {
         <T as AsCompositeIndex>::as_composite(self)
     }
 }
 
 impl<T: AsCompositeIndex + ?Sized> AsCompositeIndex for &mut T {
-    fn as_composite(&self) -> CompositeIndex<'_> {
+    fn as_composite(&self) -> &CompositeIndex {
         <T as AsCompositeIndex>::as_composite(self)
     }
 }
 
-/// Reference wrapper that provides global access to nested index segments.
-#[derive(Clone, Copy)]
-pub struct CompositeIndex<'a>(&'a dyn IndexSegment);
+/// `CompositeIndex` provides an index of both commit IDs and change IDs.
+///
+/// We refer to this as a composite index because it's a composite of multiple
+/// nested index segments where each parent segment is roughly twice as large
+/// its child. segment. This provides a good balance between read and write
+/// performance.
+// Reference wrapper that provides global access to nested index segments.
+#[derive(RefCastCustom)]
+#[repr(transparent)]
+pub struct CompositeIndex(DynIndexSegment);
 
-impl<'a> CompositeIndex<'a> {
-    pub(super) fn new(segment: &'a dyn IndexSegment) -> Self {
-        CompositeIndex(segment)
-    }
+impl CompositeIndex {
+    #[ref_cast_custom]
+    pub(super) const fn new(segment: &DynIndexSegment) -> &Self;
 
     /// Iterates parent and its ancestor readonly index segments.
     pub(super) fn ancestor_files_without_local(
         &self,
-    ) -> impl Iterator<Item = &'a Arc<ReadonlyIndexSegment>> {
+    ) -> impl Iterator<Item = &Arc<ReadonlyIndexSegment>> {
         let parent_file = self.0.parent_file();
         iter::successors(parent_file, |file| file.parent_file())
     }
 
     /// Iterates self and its ancestor index segments.
-    pub(super) fn ancestor_index_segments(&self) -> impl Iterator<Item = &'a dyn IndexSegment> {
-        iter::once(self.0).chain(
+    pub(super) fn ancestor_index_segments(&self) -> impl Iterator<Item = &DynIndexSegment> {
+        iter::once(&self.0).chain(
             self.ancestor_files_without_local()
-                .map(|file| file.as_ref() as &dyn IndexSegment),
+                .map(|file| file.as_ref() as &DynIndexSegment),
         )
     }
 
@@ -115,7 +137,6 @@ impl<'a> CompositeIndex<'a> {
         let num_commits = self.num_commits();
         let mut num_merges = 0;
         let mut max_generation_number = 0;
-        let mut is_head = vec![true; num_commits as usize];
         let mut change_ids = HashSet::new();
         for pos in 0..num_commits {
             let entry = self.entry_by_pos(IndexPosition(pos));
@@ -123,12 +144,9 @@ impl<'a> CompositeIndex<'a> {
             if entry.num_parents() > 1 {
                 num_merges += 1;
             }
-            for parent_pos in entry.parent_positions() {
-                is_head[parent_pos.0 as usize] = false;
-            }
             change_ids.insert(entry.change_id());
         }
-        let num_heads = u32::try_from(is_head.iter().filter(|is_head| **is_head).count()).unwrap();
+        let num_heads = u32::try_from(self.all_heads_pos().count()).unwrap();
 
         let mut levels = self
             .ancestor_index_segments()
@@ -149,7 +167,7 @@ impl<'a> CompositeIndex<'a> {
         }
     }
 
-    pub fn entry_by_pos(&self, pos: IndexPosition) -> IndexEntry<'a> {
+    pub fn entry_by_pos(&self, pos: IndexPosition) -> IndexEntry<'_> {
         self.ancestor_index_segments()
             .find_map(|segment| {
                 u32::checked_sub(pos.0, segment.num_parent_commits())
@@ -158,9 +176,19 @@ impl<'a> CompositeIndex<'a> {
             .unwrap()
     }
 
+    pub fn entry_by_id(&self, commit_id: &CommitId) -> Option<IndexEntry<'_>> {
+        self.ancestor_index_segments().find_map(|segment| {
+            let local_pos = segment.commit_id_to_pos(commit_id)?;
+            let pos = IndexPosition(local_pos.0 + segment.num_parent_commits());
+            Some(IndexEntry::new(segment, pos, local_pos))
+        })
+    }
+
     pub fn commit_id_to_pos(&self, commit_id: &CommitId) -> Option<IndexPosition> {
-        self.ancestor_index_segments()
-            .find_map(|segment| segment.commit_id_to_pos(commit_id))
+        self.ancestor_index_segments().find_map(|segment| {
+            let LocalPosition(local_pos) = segment.commit_id_to_pos(commit_id)?;
+            Some(IndexPosition(local_pos + segment.num_parent_commits()))
+        })
     }
 
     /// Suppose the given `commit_id` exists, returns the previous and next
@@ -180,9 +208,68 @@ impl<'a> CompositeIndex<'a> {
             .unwrap()
     }
 
-    pub fn entry_by_id(&self, commit_id: &CommitId) -> Option<IndexEntry<'a>> {
-        self.commit_id_to_pos(commit_id)
-            .map(|pos| self.entry_by_pos(pos))
+    /// Suppose the given `change_id` exists, returns the minimum prefix length
+    /// to disambiguate it within all the indexed ids including hidden ones.
+    pub(super) fn shortest_unique_change_id_prefix_len(&self, change_id: &ChangeId) -> usize {
+        let (prev_id, next_id) = self.resolve_neighbor_change_ids(change_id);
+        itertools::chain(prev_id, next_id)
+            .map(|id| hex_util::common_hex_len(change_id.as_bytes(), id.as_bytes()) + 1)
+            .max()
+            .unwrap_or(0)
+    }
+
+    /// Suppose the given `change_id` exists, returns the previous and next
+    /// change ids in lexicographical order. The returned change ids may be
+    /// hidden.
+    pub(super) fn resolve_neighbor_change_ids(
+        &self,
+        change_id: &ChangeId,
+    ) -> (Option<ChangeId>, Option<ChangeId>) {
+        self.ancestor_index_segments()
+            .map(|segment| segment.resolve_neighbor_change_ids(change_id))
+            .reduce(|(acc_prev_id, acc_next_id), (prev_id, next_id)| {
+                (
+                    acc_prev_id.into_iter().chain(prev_id).max(),
+                    acc_next_id.into_iter().chain(next_id).min(),
+                )
+            })
+            .unwrap()
+    }
+
+    /// Resolves the given change id `prefix` to the associated entries. The
+    /// returned entries may be hidden.
+    ///
+    /// The returned index positions are sorted in ascending order.
+    pub(super) fn resolve_change_id_prefix(
+        &self,
+        prefix: &HexPrefix,
+    ) -> PrefixResolution<(ChangeId, SmallIndexPositionsVec)> {
+        use PrefixResolution::*;
+        self.ancestor_index_segments()
+            .fold(NoMatch, |acc_match, segment| {
+                if acc_match == AmbiguousMatch {
+                    return acc_match; // avoid checking the parent file(s)
+                }
+                let to_global_pos = {
+                    let num_parent_commits = segment.num_parent_commits();
+                    move |LocalPosition(pos)| IndexPosition(pos + num_parent_commits)
+                };
+                // Similar to PrefixResolution::plus(), but merges matches of the same id.
+                match (acc_match, segment.resolve_change_id_prefix(prefix)) {
+                    (NoMatch, local_match) => local_match.map(|(id, positions)| {
+                        (id, positions.into_iter().map(to_global_pos).collect())
+                    }),
+                    (acc_match, NoMatch) => acc_match,
+                    (AmbiguousMatch, _) => AmbiguousMatch,
+                    (_, AmbiguousMatch) => AmbiguousMatch,
+                    (SingleMatch((id1, _)), SingleMatch((id2, _))) if id1 != id2 => AmbiguousMatch,
+                    (SingleMatch((id, mut acc_positions)), SingleMatch((_, local_positions))) => {
+                        acc_positions
+                            .insert_many(0, local_positions.into_iter().map(to_global_pos));
+                        SingleMatch((id, acc_positions))
+                    }
+                }
+            })
     }
 
     pub(super) fn is_ancestor_pos(
@@ -252,13 +339,30 @@ impl<'a> CompositeIndex<'a> {
         self.heads_pos(result)
     }
 
-    pub fn walk_revs(&self, wanted: &[IndexPosition], unwanted: &[IndexPosition]) -> RevWalk<'a> {
-        let mut rev_walk = RevWalk::new(*self);
-        rev_walk.extend_wanted(wanted.iter().copied());
-        rev_walk.extend_unwanted(unwanted.iter().copied());
-        rev_walk
+    pub(super) fn all_heads(&self) -> impl Iterator<Item = CommitId> + '_ {
+        self.all_heads_pos()
+            .map(move |pos| self.entry_by_pos(pos).commit_id())
     }
 
+    pub(super) fn all_heads_pos(&self) -> impl Iterator<Item = IndexPosition> {
+        // TODO: can be optimized to use bit vec and leading/trailing_ones()
+        let num_commits = self.num_commits();
+        let mut not_head: Vec<bool> = vec![false; num_commits as usize];
+        for pos in 0..num_commits {
+            let entry = self.entry_by_pos(IndexPosition(pos));
+            for IndexPosition(parent_pos) in entry.parent_positions() {
+                not_head[parent_pos as usize] = true;
+            }
+        }
+        not_head
+            .into_iter()
+            .enumerate()
+            .filter(|&(_, b)| !b)
+            .map(|(i, _)| IndexPosition(u32::try_from(i).unwrap()))
+    }
+
+    /// Returns the subset of positions in `candidate_positions` which refer to
+    /// entries that are heads in the repository.
     pub fn heads_pos(
         &self,
         mut candidate_positions: BTreeSet<IndexPosition>,
@@ -297,19 +401,20 @@ impl<'a> CompositeIndex<'a> {
         &self,
         expression: &ResolvedExpression,
         store: &Arc<Store>,
-    ) -> Result<Box<dyn Revset<'a> + 'a>, RevsetEvaluationError> {
-        let revset_impl = default_revset_engine::evaluate(expression, store, *self)?;
+    ) -> Result<Box<dyn Revset + '_>, RevsetEvaluationError> {
+        let revset_impl = revset_engine::evaluate(expression, store, self)?;
         Ok(Box::new(revset_impl))
     }
 }
 
-impl AsCompositeIndex for CompositeIndex<'_> {
-    fn as_composite(&self) -> CompositeIndex<'_> {
-        *self
+impl AsCompositeIndex for CompositeIndex {
+    fn as_composite(&self) -> &CompositeIndex {
+        self
     }
 }
 
-impl Index for CompositeIndex<'_> {
+// In revset engine, we need to convert &CompositeIndex to &dyn Index.
+impl Index for &CompositeIndex {
     /// Suppose the given `commit_id` exists, returns the minimum prefix length
     /// to disambiguate it. The length to be returned is a number of hexadecimal
     /// digits.
@@ -319,7 +424,7 @@ impl Index for CompositeIndex<'_> {
     fn shortest_unique_commit_id_prefix_len(&self, commit_id: &CommitId) -> usize {
         let (prev_id, next_id) = self.resolve_neighbor_commit_ids(commit_id);
         itertools::chain(prev_id, next_id)
-            .map(|id| backend::common_hex_len(commit_id.as_bytes(), id.as_bytes()) + 1)
+            .map(|id| hex_util::common_hex_len(commit_id.as_bytes(), id.as_bytes()) + 1)
             .max()
             .unwrap_or(0)
     }
@@ -361,6 +466,12 @@ impl Index for CompositeIndex<'_> {
             .collect()
     }
 
+    fn all_heads_for_gc(
+        &self,
+    ) -> Result<Box<dyn Iterator<Item = CommitId> + '_>, AllHeadsForGcUnsupported> {
+        Ok(Box::new(self.all_heads()))
+    }
+
     fn heads(&self, candidate_ids: &mut dyn Iterator<Item = &CommitId>) -> Vec<CommitId> {
         let candidate_positions: BTreeSet<_> = candidate_ids
             .map(|id| self.commit_id_to_pos(id).unwrap())
@@ -372,22 +483,77 @@ impl Index for CompositeIndex<'_> {
             .collect()
     }
 
-    /// Parents before children
-    fn topo_order(&self, input: &mut dyn Iterator<Item = &CommitId>) -> Vec<CommitId> {
-        let mut ids = input.cloned().collect_vec();
-        ids.sort_by_cached_key(|id| self.commit_id_to_pos(id).unwrap());
-        ids
-    }
-
     fn evaluate_revset<'index>(
         &'index self,
         expression: &ResolvedExpression,
         store: &Arc<Store>,
-    ) -> Result<Box<dyn Revset<'index> + 'index>, RevsetEvaluationError> {
+    ) -> Result<Box<dyn Revset + 'index>, RevsetEvaluationError> {
         CompositeIndex::evaluate_revset(self, expression, store)
     }
 }
 
+pub(super) struct ChangeIdIndexImpl<I> {
+    index: I,
+    reachable_set: Mutex<AncestorsBitSet>,
+}
+
+impl<I: AsCompositeIndex> ChangeIdIndexImpl<I> {
+    pub fn new(index: I, heads: &mut dyn Iterator<Item = &CommitId>) -> ChangeIdIndexImpl<I> {
+        let composite = index.as_composite();
+        let mut reachable_set = AncestorsBitSet::with_capacity(composite.num_commits());
+        for id in heads {
+            reachable_set.add_head(composite.commit_id_to_pos(id).unwrap());
+        }
+        ChangeIdIndexImpl {
+            index,
+            reachable_set: Mutex::new(reachable_set),
+        }
+    }
+}
+
+impl<I: AsCompositeIndex + Send + Sync> ChangeIdIndex for ChangeIdIndexImpl<I> {
+    // Resolves change id prefix among all ids, then filters out hidden
+    // entries.
+    //
+    // If `SingleMatch` is returned, the commits including in the set are all
+    // visible. `AmbiguousMatch` may be returned even if the prefix is unique
+    // within the visible entries.
+    fn resolve_prefix(&self, prefix: &HexPrefix) -> PrefixResolution<Vec<CommitId>> {
+        let index = self.index.as_composite();
+        match index.resolve_change_id_prefix(prefix) {
+            PrefixResolution::NoMatch => PrefixResolution::NoMatch,
+            PrefixResolution::SingleMatch((_change_id, positions)) => {
+                debug_assert!(positions.iter().tuple_windows().all(|(a, b)| a < b));
+                let mut reachable_set = self.reachable_set.lock().unwrap();
+                reachable_set.visit_until(index, *positions.first().unwrap());
+                let reachable_commit_ids = positions
+                    .iter()
+                    .filter(|&&pos| reachable_set.contains(pos))
+                    .map(|&pos| index.entry_by_pos(pos).commit_id())
+                    .collect_vec();
+                if reachable_commit_ids.is_empty() {
+                    PrefixResolution::NoMatch
+                } else {
+                    PrefixResolution::SingleMatch(reachable_commit_ids)
+                }
+            }
+            PrefixResolution::AmbiguousMatch => PrefixResolution::AmbiguousMatch,
+        }
+    }
+
+    // Calculates the shortest prefix length of the given `change_id` among all
+    // IDs, including hidden entries.
+    //
+    // The returned length is usually a few digits longer than the minimum
+    // length necessary to disambiguate within the visible entries since hidden
+    // entries are also considered when determining the prefix length.
+    fn shortest_unique_prefix_len(&self, change_id: &ChangeId) -> usize {
+        self.index
+            .as_composite()
+            .shortest_unique_change_id_prefix_len(change_id)
+    }
+}
+
 pub struct IndexLevelStats {
     pub num_commits: u32,
     pub name: Option<String>,
