@@ -22,8 +22,6 @@ use std::slice;
 
 use itertools::Itertools;
 
-use crate::nightly_shims::BTreeMapExt;
-
 pub fn find_line_ranges(text: &[u8]) -> Vec<Range<usize>> {
     let mut ranges = vec![];
     let mut start = 0;
@@ -185,7 +183,7 @@ pub(crate) fn unchanged_ranges(
 
     let max_occurrences = 100;
     let mut left_histogram = Histogram::calculate(left, left_ranges, max_occurrences);
-    if *left_histogram.count_to_words.first_key().unwrap() > max_occurrences {
+    if *left_histogram.count_to_words.keys().next().unwrap() > max_occurrences {
         // If there are very many occurrences of all words, then we just give up.
         return vec![];
     }
@@ -195,7 +193,11 @@ pub(crate) fn unchanged_ranges(
     // the LCS.
     let mut uncommon_shared_words = vec![];
     while !left_histogram.count_to_words.is_empty() && uncommon_shared_words.is_empty() {
-        let left_words = left_histogram.count_to_words.pop_first_value().unwrap();
+        let left_words = left_histogram
+            .count_to_words
+            .first_entry()
+            .map(|x| x.remove())
+            .unwrap();
         for left_word in left_words {
             if right_histogram.word_to_positions.contains_key(left_word) {
                 uncommon_shared_words.push(left_word);
