@@ -18,6 +18,13 @@
 #![deny(unused_must_use)]
 #![forbid(unsafe_code)]
 
+// Needed so that proc macros can be used inside jj_lib and by external crates
+// that depend on it.
+// See:
+// - https://github.com/rust-lang/rust/issues/54647#issuecomment-432015102
+// - https://github.com/rust-lang/rust/issues/54363
+extern crate self as jj_lib;
+
 #[macro_use]
 pub mod content_hash;
 
@@ -29,13 +36,18 @@ pub mod dag_walk;
 pub mod default_index;
 pub mod default_submodule_store;
 pub mod diff;
+pub mod dsl_util;
+pub mod extensions_map;
 pub mod file_util;
 pub mod files;
+pub mod fileset;
+mod fileset_parser;
 pub mod fmt_util;
 pub mod fsmonitor;
 pub mod git;
 pub mod git_backend;
 pub mod gitignore;
+pub mod gpg_signing;
 pub mod hex_util;
 pub mod id_prefix;
 pub mod index;
@@ -62,6 +74,7 @@ pub mod settings;
 pub mod signing;
 pub mod simple_op_heads_store;
 pub mod simple_op_store;
+pub mod ssh_signing;
 pub mod stacked_table;
 pub mod store;
 pub mod str_util;
