@@ -29,7 +29,7 @@ use smallvec::{smallvec_inline, SmallVec};
 
 use crate::backend;
 use crate::backend::{BackendError, FileId, TreeId, TreeValue};
-use crate::content_hash::ContentHash;
+use crate::content_hash::{ContentHash, DigestUpdate};
 use crate::object_id::ObjectId;
 use crate::repo_path::RepoPath;
 use crate::store::Store;
@@ -92,8 +92,8 @@ where
     } else if counts.len() == 2 {
         // All sides made the same change.
         // This matches what Git and Mercurial do (in the 3-way case at least), but not
-        // what Darcs and Pijul do. It means that repeated 3-way merging of multiple
-        // trees may give different results depending on the order of merging.
+        // what Darcs does. It means that repeated 3-way merging of multiple trees may
+        // give different results depending on the order of merging.
         // TODO: Consider removing this special case, making the algorithm more strict,
         // and maybe add a more lenient version that is used when the user explicitly
         // asks for conflict resolution.
@@ -402,6 +402,11 @@ impl<T> Merge<Option<T>> {
         !self.is_absent()
     }
 
+    /// Returns the value if this is present and non-conflicting.
+    pub fn as_normal(&self) -> Option<&T> {
+        self.as_resolved()?.as_ref()
+    }
+
     /// Creates lists of `removes` and `adds` from a `Merge` by dropping
     /// `None` values. Note that the conversion is lossy: the order of `None`
     /// values is not preserved when converting back to a `Merge`.
@@ -452,7 +457,7 @@ impl<T> Merge<Merge<T>> {
 }
 
 impl<T: ContentHash> ContentHash for Merge<T> {
-    fn hash(&self, state: &mut impl digest::Update) {
+    fn hash(&self, state: &mut impl DigestUpdate) {
         self.values.hash(state)
     }
 }
