@@ -164,10 +164,10 @@ impl UserSettings {
         self.config.get_string("user.email").unwrap_or_default()
     }
 
-    pub fn fsmonitor_kind(&self) -> Result<Option<FsmonitorKind>, config::ConfigError> {
+    pub fn fsmonitor_kind(&self) -> Result<FsmonitorKind, config::ConfigError> {
         match self.config.get_string("core.fsmonitor") {
-            Ok(fsmonitor_kind) => Ok(Some(fsmonitor_kind.parse()?)),
-            Err(config::ConfigError::NotFound(_)) => Ok(None),
+            Ok(fsmonitor_kind) => Ok(fsmonitor_kind.parse()?),
+            Err(config::ConfigError::NotFound(_)) => Ok(FsmonitorKind::None),
             Err(err) => Err(err),
         }
     }
@@ -183,7 +183,7 @@ impl UserSettings {
     pub fn operation_hostname(&self) -> String {
         self.config
             .get_string("operation.hostname")
-            .unwrap_or_else(|_| whoami::hostname())
+            .unwrap_or_else(|_| whoami::fallible::hostname().expect("valid hostname"))
     }
 
     pub fn operation_username(&self) -> String {
@@ -209,9 +209,7 @@ impl UserSettings {
             // For compatibility with old config files (<0.8.0)
             self.config
                 .get_string("ui.default-revset")
-                .unwrap_or_else(|_| {
-                    "@ | ancestors(immutable_heads().., 2) | heads(immutable_heads())".to_string()
-                })
+                .unwrap_or_else(|_| "@ | ancestors(immutable_heads().., 2) | trunk()".to_string())
         })
     }
 
@@ -230,10 +228,6 @@ impl UserSettings {
             .unwrap_or(false)
     }
 
-    pub fn diff_instructions(&self) -> bool {
-        self.config.get_bool("ui.diff-instructions").unwrap_or(true)
-    }
-
     pub fn config(&self) -> &config::Config {
         &self.config
     }
@@ -248,6 +242,22 @@ impl UserSettings {
             .unwrap_or_else(|_| "curved".to_string())
     }
 
+    pub fn commit_node_template(&self) -> String {
+        self.node_template_for_key(
+            "templates.log_node",
+            r#"if(self, if(current_working_copy, "@", "◉"), "◌")"#,
+            r#"if(self, if(current_working_copy, "@", "o"), ".")"#,
+        )
+    }
+
+    pub fn op_node_template(&self) -> String {
+        self.node_template_for_key(
+            "templates.op_log_node",
+            r#"if(current_operation, "@", "◉")"#,
+            r#"if(current_operation, "@", "o")"#,
+        )
+    }
+
     pub fn max_new_file_size(&self) -> Result<u64, config::ConfigError> {
         let cfg = self
             .config
@@ -270,6 +280,14 @@ impl UserSettings {
     pub fn sign_settings(&self) -> SignSettings {
         SignSettings::from_settings(self)
     }
+
+    fn node_template_for_key(&self, key: &str, fallback: &str, ascii_fallback: &str) -> String {
+        let symbol = self.config.get_string(key);
+        match self.graph_style().as_str() {
+            "ascii" | "ascii-large" => symbol.unwrap_or_else(|_| ascii_fallback.to_owned()),
+            _ => symbol.unwrap_or_else(|_| fallback.to_owned()),
+        }
+    }
 }
 
 /// This Rng uses interior mutability to allow generating random values using an
@@ -313,7 +331,8 @@ impl<T> ConfigResultExt<T> for Result<T, config::ConfigError> {
 }
 
 /// A size in bytes optionally formatted/serialized with binary prefixes
-#[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, serde::Deserialize)]
+#[serde(try_from = "String")]
 pub struct HumanByteSize(pub u64);
 
 impl std::fmt::Display for HumanByteSize {
@@ -323,43 +342,18 @@ impl std::fmt::Display for HumanByteSize {
     }
 }
 
-impl<'de> serde::Deserialize<'de> for HumanByteSize {
-    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
-    where
-        D: serde::Deserializer<'de>,
-    {
-        use serde::de::Error;
+impl TryFrom<String> for HumanByteSize {
+    type Error = String;
 
-        struct Visitor;
-
-        impl<'de> serde::de::Visitor<'de> for Visitor {
-            type Value = HumanByteSize;
-
-            fn expecting(&self, formatter: &mut std::fmt::Formatter) -> std::fmt::Result {
-                write!(formatter, "a size in bytes with an optional binary unit")
-            }
-
-            fn visit_u64<E>(self, v: u64) -> Result<Self::Value, E>
-            where
-                E: Error,
-            {
-                Ok(HumanByteSize(v))
-            }
-
-            fn visit_str<E>(self, v: &str) -> Result<Self::Value, E>
-            where
-                E: Error,
-            {
-                let bytes = parse_human_byte_size(v).map_err(Error::custom)?;
+    fn try_from(value: String) -> Result<Self, Self::Error> {
+        let res = value.parse::<u64>();
+        match res {
+            Ok(bytes) => Ok(HumanByteSize(bytes)),
+            Err(_) => {
+                let bytes = parse_human_byte_size(&value)?;
                 Ok(HumanByteSize(bytes))
             }
         }
-
-        if deserializer.is_human_readable() {
-            deserializer.deserialize_any(Visitor)
-        } else {
-            deserializer.deserialize_u64(Visitor)
-        }
     }
 }
 
