@@ -14,12 +14,13 @@
 
 #![allow(missing_docs)]
 
+use std::any::Any;
 use std::fmt::{Debug, Formatter};
 use std::fs;
 use std::path::{Path, PathBuf};
 
-use crate::backend::ObjectId;
 use crate::lock::FileLock;
+use crate::object_id::ObjectId;
 use crate::op_heads_store::{OpHeadsStore, OpHeadsStoreLock};
 use crate::op_store::OperationId;
 
@@ -71,11 +72,16 @@ struct SimpleOpHeadsStoreLock {
 impl OpHeadsStoreLock for SimpleOpHeadsStoreLock {}
 
 impl OpHeadsStore for SimpleOpHeadsStore {
+    fn as_any(&self) -> &dyn Any {
+        self
+    }
+
     fn name(&self) -> &str {
         Self::name()
     }
 
     fn update_op_heads(&self, old_ids: &[OperationId], new_id: &OperationId) {
+        assert!(!old_ids.contains(new_id));
         self.add_op_head(new_id);
         for old_id in old_ids {
             self.remove_op_head(old_id)
