@@ -49,8 +49,6 @@ pub enum WorkspaceInitError {
     DestinationExists(PathBuf),
     #[error("Repo path could not be interpreted as Unicode text")]
     NonUnicodePath,
-    #[error("Path {0} does not exist")]
-    PathNotFound(PathBuf),
     #[error(transparent)]
     CheckOutCommit(#[from] CheckOutCommitError),
     #[error(transparent)]
@@ -74,11 +72,15 @@ pub enum WorkspaceLoadError {
     #[error("Repo path could not be interpreted as Unicode text")]
     NonUnicodePath,
     #[error(transparent)]
+    WorkingCopyState(#[from] WorkingCopyStateError),
+    #[error(transparent)]
     Path(#[from] PathError),
 }
 
-/// Represents a workspace, i.e. what's typically the .jj/ directory and its
-/// parent.
+/// Represents the combination of a repo and working copy, i.e. what's typically
+/// the .jj/ directory and its parent. See
+/// <https://github.com/martinvonz/jj/blob/main/docs/working-copy.md#workspaces>
+/// for more information.
 pub struct Workspace {
     // Path to the workspace root (typically the parent of a .jj/ directory), which is where
     // working copy files live.
@@ -192,22 +194,14 @@ impl Workspace {
     }
 
     /// Initializes a workspace with an existing Git repo at the specified path.
+    ///
+    /// The `git_repo_path` usually ends with `.git`. It's the path to the Git
+    /// repo directory, not the working directory.
     pub fn init_external_git(
         user_settings: &UserSettings,
         workspace_root: &Path,
         git_repo_path: &Path,
     ) -> Result<(Self, Arc<ReadonlyRepo>), WorkspaceInitError> {
-        let mut git_repo_path = git_repo_path
-            .canonicalize()
-            .map_err(|_| WorkspaceInitError::PathNotFound(git_repo_path.to_path_buf()))?;
-        if !git_repo_path.ends_with(".git") {
-            git_repo_path.push(".git");
-
-            if !git_repo_path.exists() {
-                git_repo_path.pop();
-            }
-        }
-
         let backend_initializer =
             |settings: &UserSettings, store_path: &Path| -> Result<Box<dyn Backend>, _> {
                 // If the git repo is inside the workspace, use a relative path to it so the
@@ -217,7 +211,7 @@ impl Workspace {
                 // Workspace::new(), but it's not yet here.
                 let store_relative_git_repo_path = match (
                     workspace_root.canonicalize(),
-                    canonicalize_git_repo_path(&git_repo_path),
+                    canonicalize_git_repo_path(git_repo_path),
                 ) {
                     (Ok(workspace_root), Ok(git_repo_path))
                         if git_repo_path.starts_with(&workspace_root) =>
@@ -340,7 +334,7 @@ impl Workspace {
         user_settings: &UserSettings,
         workspace_path: &Path,
         store_factories: &StoreFactories,
-        working_copy_factories: &HashMap<String, Box<dyn WorkingCopyFactory>>,
+        working_copy_factories: &WorkingCopyFactories,
     ) -> Result<Self, WorkspaceLoadError> {
         let loader = WorkspaceLoader::init(workspace_path)?;
         let workspace = loader.load(user_settings, store_factories, working_copy_factories)?;
@@ -433,6 +427,8 @@ pub struct WorkspaceLoader {
     working_copy_state_path: PathBuf,
 }
 
+pub type WorkingCopyFactories = HashMap<String, Box<dyn WorkingCopyFactory>>;
+
 impl WorkspaceLoader {
     pub fn init(workspace_root: &Path) -> Result<Self, WorkspaceLoadError> {
         let jj_dir = workspace_root.join(".jj");
@@ -476,7 +472,7 @@ impl WorkspaceLoader {
         &self,
         user_settings: &UserSettings,
         store_factories: &StoreFactories,
-        working_copy_factories: &HashMap<String, Box<dyn WorkingCopyFactory>>,
+        working_copy_factories: &WorkingCopyFactories,
     ) -> Result<Workspace, WorkspaceLoadError> {
         let repo_loader = RepoLoader::init(user_settings, &self.repo_dir, store_factories)?;
         let working_copy = self.load_working_copy(repo_loader.store(), working_copy_factories)?;
@@ -486,7 +482,7 @@ impl WorkspaceLoader {
 
     pub fn get_working_copy_factory<'a>(
         &self,
-        working_copy_factories: &'a HashMap<String, Box<dyn WorkingCopyFactory>>,
+        working_copy_factories: &'a WorkingCopyFactories,
     ) -> Result<&'a dyn WorkingCopyFactory, StoreLoadError> {
         // For compatibility with existing repos. TODO: Delete default in 0.17+
         let working_copy_type = read_store_type_compat(
@@ -508,19 +504,20 @@ impl WorkspaceLoader {
     fn load_working_copy(
         &self,
         store: &Arc<Store>,
-        working_copy_factories: &HashMap<String, Box<dyn WorkingCopyFactory>>,
-    ) -> Result<Box<dyn WorkingCopy>, StoreLoadError> {
+        working_copy_factories: &WorkingCopyFactories,
+    ) -> Result<Box<dyn WorkingCopy>, WorkspaceLoadError> {
         let working_copy_factory = self.get_working_copy_factory(working_copy_factories)?;
-        Ok(working_copy_factory.load_working_copy(
+        let working_copy = working_copy_factory.load_working_copy(
             store.clone(),
             self.workspace_root.to_owned(),
             self.working_copy_state_path.to_owned(),
-        ))
+        )?;
+        Ok(working_copy)
     }
 }
 
-pub fn default_working_copy_factories() -> HashMap<String, Box<dyn WorkingCopyFactory>> {
-    let mut factories: HashMap<String, Box<dyn WorkingCopyFactory>> = HashMap::new();
+pub fn default_working_copy_factories() -> WorkingCopyFactories {
+    let mut factories = WorkingCopyFactories::new();
     factories.insert(
         LocalWorkingCopy::name().to_owned(),
         Box::new(LocalWorkingCopyFactory {}),
