@@ -12,12 +12,6 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-// TODO: Remove when MSRV passes 1.72
-// https://github.com/frondeus/test-case/issues/126#issuecomment-1635916592
-#![allow(clippy::items_after_test_module)]
-
-use std::fs::OpenOptions;
-use std::io::Write;
 #[cfg(unix)]
 use std::os::unix::fs::PermissionsExt;
 #[cfg(unix)]
@@ -26,11 +20,12 @@ use std::path::Path;
 use std::sync::Arc;
 
 use itertools::Itertools;
-use jj_lib::backend::{MergedTreeId, ObjectId, TreeId, TreeValue};
+use jj_lib::backend::{MergedTreeId, TreeId, TreeValue};
+use jj_lib::file_util::{check_symlink_support, try_symlink};
 use jj_lib::fsmonitor::FsmonitorKind;
 use jj_lib::local_working_copy::LocalWorkingCopy;
-use jj_lib::merge::Merge;
-use jj_lib::merged_tree::MergedTreeBuilder;
+use jj_lib::merge::{Merge, MergedTreeValue};
+use jj_lib::merged_tree::{MergedTree, MergedTreeBuilder};
 use jj_lib::op_store::{OperationId, WorkspaceId};
 use jj_lib::repo::{ReadonlyRepo, Repo};
 use jj_lib::repo_path::{RepoPath, RepoPathBuf, RepoPathComponent};
@@ -46,6 +41,12 @@ fn to_owned_path_vec(paths: &[&RepoPath]) -> Vec<RepoPathBuf> {
     paths.iter().map(|&path| path.to_owned()).collect()
 }
 
+fn tree_entries(tree: &MergedTree) -> Vec<(RepoPathBuf, Option<MergedTreeValue>)> {
+    tree.entries()
+        .map(|(path, result)| (path, result.ok()))
+        .collect_vec()
+}
+
 #[test]
 fn test_root() {
     // Test that the working copy is clean and empty after init.
@@ -86,7 +87,6 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
         // Executable, but same content as Normal, to test transition where only the bit changed
         ExecutableNormalContent,
         Conflict,
-        #[cfg_attr(windows, allow(dead_code))]
         Symlink,
         Tree,
         GitSubmodule,
@@ -176,7 +176,6 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
         Kind::Conflict,
         Kind::Tree,
     ];
-    #[cfg(unix)]
     kinds.push(Kind::Symlink);
     if backend == TestRepoBackend::Git {
         kinds.push(Kind::GitSubmodule);
@@ -250,10 +249,12 @@ fn test_checkout_file_transitions(backend: TestRepoBackend) {
             Kind::Symlink => {
                 assert!(maybe_metadata.is_ok(), "{path:?} should exist");
                 let metadata = maybe_metadata.unwrap();
-                assert!(
-                    metadata.file_type().is_symlink(),
-                    "{path:?} should be a symlink"
-                );
+                if check_symlink_support().unwrap_or(false) {
+                    assert!(
+                        metadata.file_type().is_symlink(),
+                        "{path:?} should be a symlink"
+                    );
+                }
             }
             Kind::Tree => {
                 assert!(maybe_metadata.is_ok(), "{path:?} should exist");
@@ -414,10 +415,12 @@ fn test_reset() {
     let gitignore_path = RepoPath::from_internal_string(".gitignore");
 
     let tree_without_file = create_tree(repo, &[(gitignore_path, "ignored\n")]);
+    let commit_without_file = commit_with_tree(repo.store(), tree_without_file.id().clone());
     let tree_with_file = create_tree(
         repo,
         &[(gitignore_path, "ignored\n"), (ignored_path, "code")],
     );
+    let commit_with_file = commit_with_tree(repo.store(), tree_with_file.id().clone());
 
     let ws = &mut test_workspace.workspace;
     let commit = commit_with_tree(repo.store(), tree_with_file.id());
@@ -432,7 +435,7 @@ fn test_reset() {
     // but it should not be in the tree state, and it should not get added when we
     // commit the working copy (because it's ignored).
     let mut locked_ws = ws.start_working_copy_mutation().unwrap();
-    locked_ws.locked_wc().reset(&tree_without_file).unwrap();
+    locked_ws.locked_wc().reset(&commit_without_file).unwrap();
     locked_ws.finish(op_id.clone()).unwrap();
     assert!(ignored_path.to_fs_path(&workspace_root).is_file());
     let wc: &LocalWorkingCopy = ws.working_copy().as_any().downcast_ref().unwrap();
@@ -444,7 +447,7 @@ fn test_reset() {
     // tracked. The file should become tracked (even though it's ignored).
     let ws = &mut test_workspace.workspace;
     let mut locked_ws = ws.start_working_copy_mutation().unwrap();
-    locked_ws.locked_wc().reset(&tree_with_file).unwrap();
+    locked_ws.locked_wc().reset(&commit_with_file).unwrap();
     locked_ws.finish(op_id.clone()).unwrap();
     assert!(ignored_path.to_fs_path(&workspace_root).is_file());
     let wc: &LocalWorkingCopy = ws.working_copy().as_any().downcast_ref().unwrap();
@@ -517,16 +520,7 @@ fn test_snapshot_racy_timestamps() {
     let file_path = workspace_root.join("file");
     let mut previous_tree_id = repo.store().empty_merged_tree_id();
     for i in 0..100 {
-        {
-            // https://github.com/rust-lang/rust-clippy/issues/9778
-            #[allow(clippy::needless_borrow)]
-            let mut file = OpenOptions::new()
-                .create(true)
-                .write(true)
-                .open(&file_path)
-                .unwrap();
-            file.write_all(format!("contents {i}").as_bytes()).unwrap();
-        }
+        std::fs::write(&file_path, format!("contents {i}").as_bytes()).unwrap();
         let mut locked_ws = test_workspace
             .workspace
             .start_working_copy_mutation()
@@ -681,10 +675,7 @@ fn test_gitignores_in_ignored_dir() {
     testutils::write_working_copy_file(&workspace_root, ignored_path, "contents");
 
     let new_tree = test_workspace.snapshot().unwrap();
-    assert_eq!(
-        new_tree.entries().collect_vec(),
-        tree1.entries().collect_vec()
-    );
+    assert_eq!(tree_entries(&new_tree), tree_entries(&tree1));
 
     // The nested .gitignore is ignored even if it's tracked
     let tree2 = create_tree(
@@ -694,18 +685,16 @@ fn test_gitignores_in_ignored_dir() {
             (nested_gitignore_path, "!file\n"),
         ],
     );
+    let commit2 = commit_with_tree(test_workspace.repo.store(), tree2.id().clone());
     let mut locked_ws = test_workspace
         .workspace
         .start_working_copy_mutation()
         .unwrap();
-    locked_ws.locked_wc().reset(&tree2).unwrap();
+    locked_ws.locked_wc().reset(&commit2).unwrap();
     locked_ws.finish(OperationId::from_hex("abc123")).unwrap();
 
     let new_tree = test_workspace.snapshot().unwrap();
-    assert_eq!(
-        new_tree.entries().collect_vec(),
-        tree2.entries().collect_vec()
-    );
+    assert_eq!(tree_entries(&new_tree), tree_entries(&tree2));
 }
 
 #[test]
@@ -750,19 +739,54 @@ fn test_gitignores_ignored_directory_already_tracked() {
     let workspace_root = test_workspace.workspace.workspace_root().clone();
     let repo = test_workspace.repo.clone();
 
+    #[derive(Clone, Copy, Debug, Eq, PartialEq)]
+    enum Kind {
+        Normal,
+        Executable,
+        Symlink,
+    }
+    let create_tree_with_kind = |path_contents: &[(&RepoPath, Kind, &str)]| {
+        let store = repo.store();
+        let mut tree_builder = store.tree_builder(store.empty_tree_id().clone());
+        for (path, kind, contents) in path_contents {
+            match kind {
+                Kind::Normal => {
+                    testutils::write_normal_file(&mut tree_builder, path, contents);
+                }
+                Kind::Executable => {
+                    testutils::write_executable_file(&mut tree_builder, path, contents);
+                }
+                Kind::Symlink => {
+                    testutils::write_symlink(&mut tree_builder, path, contents);
+                }
+            }
+        }
+        let id = tree_builder.write_tree();
+        MergedTree::legacy(store.get_tree(RepoPath::root(), &id).unwrap())
+    };
+
     let gitignore_path = RepoPath::from_internal_string(".gitignore");
-    let unchanged_path = RepoPath::from_internal_string("ignored/unchanged");
-    let modified_path = RepoPath::from_internal_string("ignored/modified");
-    let deleted_path = RepoPath::from_internal_string("ignored/deleted");
-    let tree = create_tree(
-        &repo,
-        &[
-            (gitignore_path, "/ignored/\n"),
-            (unchanged_path, "contents"),
-            (modified_path, "contents"),
-            (deleted_path, "contents"),
-        ],
-    );
+    let unchanged_normal_path = RepoPath::from_internal_string("ignored/unchanged_normal");
+    let modified_normal_path = RepoPath::from_internal_string("ignored/modified_normal");
+    let deleted_normal_path = RepoPath::from_internal_string("ignored/deleted_normal");
+    let unchanged_executable_path = RepoPath::from_internal_string("ignored/unchanged_executable");
+    let modified_executable_path = RepoPath::from_internal_string("ignored/modified_executable");
+    let deleted_executable_path = RepoPath::from_internal_string("ignored/deleted_executable");
+    let unchanged_symlink_path = RepoPath::from_internal_string("ignored/unchanged_symlink");
+    let modified_symlink_path = RepoPath::from_internal_string("ignored/modified_symlink");
+    let deleted_symlink_path = RepoPath::from_internal_string("ignored/deleted_symlink");
+    let tree = create_tree_with_kind(&[
+        (gitignore_path, Kind::Normal, "/ignored/\n"),
+        (unchanged_normal_path, Kind::Normal, "contents"),
+        (modified_normal_path, Kind::Normal, "contents"),
+        (deleted_normal_path, Kind::Normal, "contents"),
+        (unchanged_executable_path, Kind::Executable, "contents"),
+        (modified_executable_path, Kind::Executable, "contents"),
+        (deleted_executable_path, Kind::Executable, "contents"),
+        (unchanged_symlink_path, Kind::Symlink, "contents"),
+        (modified_symlink_path, Kind::Symlink, "contents"),
+        (deleted_symlink_path, Kind::Symlink, "contents"),
+    ]);
     let commit = commit_with_tree(repo.store(), tree.id());
 
     // Check out the tree with the files in `ignored/`
@@ -772,21 +796,33 @@ fn test_gitignores_ignored_directory_already_tracked() {
     // Make some changes inside the ignored directory and check that they are
     // detected when we snapshot. The files that are still there should not be
     // deleted from the resulting tree.
-    std::fs::write(modified_path.to_fs_path(&workspace_root), "modified").unwrap();
-    std::fs::remove_file(deleted_path.to_fs_path(&workspace_root)).unwrap();
+    std::fs::write(modified_normal_path.to_fs_path(&workspace_root), "modified").unwrap();
+    std::fs::remove_file(deleted_normal_path.to_fs_path(&workspace_root)).unwrap();
+    std::fs::write(
+        modified_executable_path.to_fs_path(&workspace_root),
+        "modified",
+    )
+    .unwrap();
+    std::fs::remove_file(deleted_executable_path.to_fs_path(&workspace_root)).unwrap();
+    let fs_path = modified_symlink_path.to_fs_path(&workspace_root);
+    std::fs::remove_file(&fs_path).unwrap();
+    if check_symlink_support().unwrap_or(false) {
+        try_symlink("modified", &fs_path).unwrap();
+    } else {
+        std::fs::write(fs_path, "modified").unwrap();
+    }
+    std::fs::remove_file(deleted_symlink_path.to_fs_path(&workspace_root)).unwrap();
     let new_tree = test_workspace.snapshot().unwrap();
-    let expected_tree = create_tree(
-        &repo,
-        &[
-            (gitignore_path, "/ignored/\n"),
-            (unchanged_path, "contents"),
-            (modified_path, "modified"),
-        ],
-    );
-    assert_eq!(
-        new_tree.entries().collect_vec(),
-        expected_tree.entries().collect_vec()
-    );
+    let expected_tree = create_tree_with_kind(&[
+        (gitignore_path, Kind::Normal, "/ignored/\n"),
+        (unchanged_normal_path, Kind::Normal, "contents"),
+        (modified_normal_path, Kind::Normal, "modified"),
+        (unchanged_executable_path, Kind::Executable, "contents"),
+        (modified_executable_path, Kind::Executable, "modified"),
+        (unchanged_symlink_path, Kind::Symlink, "contents"),
+        (modified_symlink_path, Kind::Symlink, "modified"),
+    ]);
+    assert_eq!(tree_entries(&new_tree), tree_entries(&expected_tree));
 }
 
 #[test]
@@ -883,7 +919,6 @@ fn test_gitsubmodule() {
     );
 }
 
-#[cfg(unix)]
 #[test]
 fn test_existing_directory_symlink() {
     let settings = testutils::user_settings();
@@ -893,7 +928,12 @@ fn test_existing_directory_symlink() {
 
     // Creates a symlink in working directory, and a tree that will add a file under
     // the symlinked directory.
-    std::os::unix::fs::symlink("..", workspace_root.join("parent")).unwrap();
+    if check_symlink_support().unwrap_or(false) {
+        try_symlink("..", workspace_root.join("parent")).unwrap();
+    } else {
+        return;
+    }
+
     let file_path = RepoPath::from_internal_string("parent/escaped");
     let tree = create_tree(repo, &[(file_path, "contents")]);
     let commit = commit_with_tree(repo.store(), tree.id());
@@ -936,9 +976,9 @@ fn test_fsmonitor() {
         locked_ws
             .locked_wc()
             .snapshot(SnapshotOptions {
-                fsmonitor_kind: Some(FsmonitorKind::Test {
+                fsmonitor_kind: FsmonitorKind::Test {
                     changed_files: fs_paths,
-                }),
+                },
                 ..SnapshotOptions::empty_for_test()
             })
             .unwrap()
